@@ -24,6 +24,7 @@
 static crispy_t crispy_s = {
 	0,
 	.extautomap = 1,
+	.gamma = 9,  // default level is "OFF" for intermediate gamma levels
 	.hires = 1,
 	.smoothscaling = 1,
 	.soundfix = 1,
@@ -31,6 +32,7 @@ static crispy_t crispy_s = {
 	.truecolor = 1,
 #endif
 	.vsync = 1,
+	.widescreen = 1, // match screen by default
 };
 crispy_t *const crispy = &crispy_s;
 
