@@ -61,6 +61,7 @@ typedef struct
 	int freeaim;
 	int freelook;
 	int freelook_hh;
+	int gamma;
 	int hires;
 	int jump;
 	int leveltime;
@@ -96,6 +97,7 @@ typedef struct
 	boolean fliplevels;
 	boolean flipweapons;
 	boolean haved1e5;
+	boolean haved1e6;
 	boolean havee1m10;
 	boolean havemap33;
 	boolean havessg;
@@ -112,6 +114,7 @@ typedef struct
 	char *havenerve;
 	char *havemaster;
 	char *havesigil;
+	char *havesigil2;
 
 	const char *sdlversion;
 	const char *platform;
