@@ -28,16 +28,17 @@
 #include "m_misc.h"
 #include "w_main.h"
 #include "w_wad.h"
+#include "w_merge.h" // [crispy] W_MergeFile()
 
 extern char *iwadfile;
 
 // [crispy] auto-load SIGIL.WAD (and SIGIL_SHREDS.WAD) if available
-void D_LoadSigilWad (void)
+static boolean LoadSigilWad (const char *iwaddir, boolean pwadtexture)
 {
 	int i, j;
 	char *sigil_shreds = NULL;
 	const char *sigil_basename;
-	char *dirname, *autoload_dir;
+	char *autoload_dir;
 
 	const char *const sigil_wads[] = {
 		"SIGIL_v1_21.wad",
@@ -45,10 +46,7 @@ void D_LoadSigilWad (void)
 		"SIGIL.wad"
 	};
 
-	static const struct {
-		const char *name;
-		const char new_name[8];
-	} sigil_lumps [] = {
+	static const lump_rename_t sigil_lumps [] = {
 		{"CREDIT",   "SIGCREDI"},
 		{"HELP1",    "SIGHELP1"},
 		{"TITLEPIC", "SIGTITLE"},
@@ -61,12 +59,6 @@ void D_LoadSigilWad (void)
 		{"D_INTRO",  "D_SIGTIT"},
 	};
 
-	const char *const texture_files[] = {
-		"PNAMES",
-		"TEXTURE1",
-		"TEXTURE2",
-	};
-
 	// [crispy] don't load SIGIL.WAD if another PWAD already provides E5M1
 	i = W_CheckNumForName("E5M1");
 	if (i != -1)
@@ -76,34 +68,27 @@ void D_LoadSigilWad (void)
 		{
 			crispy->havesigil = (char *)-1;
 		}
-		return;
+		return false;
 	}
 
 	// [crispy] don't load SIGIL.WAD if SIGIL_COMPAT.WAD is already loaded
 	i = W_CheckNumForName("E3M1");
 	if (i != -1 && !strncasecmp(W_WadNameForLump(lumpinfo[i]), "SIGIL_COMPAT", 12))
 	{
-		return;
+		return false;
 	}
 
-	// [crispy] don't load SIGIL.WAD if another PWAD already modifies the texture files
-	for (i = 0; i < arrlen(texture_files); i++)
+	if (pwadtexture)
 	{
-		j = W_CheckNumForName(texture_files[i]);
-
-		if (j != -1 && !W_IsIWADLump(lumpinfo[j]))
-		{
-			return;
-		}
+		return false;
 	}
 
-	dirname = M_DirName(iwadfile);
-	sigil_shreds = M_StringJoin(dirname, DIR_SEPARATOR_S, "SIGIL_SHREDS.WAD", NULL);
+	sigil_shreds = M_StringJoin(iwaddir, DIR_SEPARATOR_S, "SIGIL_SHREDS.WAD", NULL);
 
 	// [crispy] load SIGIL.WAD
 	for (i = 0; i < arrlen(sigil_wads); i++)
 	{
-		crispy->havesigil = M_StringJoin(dirname, DIR_SEPARATOR_S, sigil_wads[i], NULL);
+		crispy->havesigil = M_StringJoin(iwaddir, DIR_SEPARATOR_S, sigil_wads[i], NULL);
 
 		if (M_FileExists(crispy->havesigil))
 		{
@@ -118,12 +103,11 @@ void D_LoadSigilWad (void)
 			break;
 		}
 	}
-	free(dirname);
 
 	if (crispy->havesigil == NULL)
 	{
 		free(sigil_shreds);
-		return;
+		return false;
 	}
 
 	printf(" [Sigil] adding %s\n", crispy->havesigil);
@@ -177,14 +161,154 @@ void D_LoadSigilWad (void)
 	{
 		if ((autoload_dir = M_GetAutoloadDir(sigil_basename, false)))
 		{
-			W_AutoLoadWADs(autoload_dir);
+			W_AutoLoadWADsRename(autoload_dir, sigil_lumps, arrlen(sigil_lumps));
 			DEH_AutoLoadPatches(autoload_dir);
 			free(autoload_dir);
 		}
 	}
 
-	// [crispy] regenerate the hashtable
-	W_GenerateHashTable();
+	return true;
+}
+
+// [crispy] auto-load Sigil II
+static boolean LoadSigil2Wad (const char *iwaddir, boolean pwadtexture)
+{
+    int i, j;
+    const char *sigil2_basename;
+    char *autoload_dir;
+
+    const char *const sigil2_wads[] = {
+        "SIGIL_II_MP3_V1_0.WAD",
+        "SIGIL_II_V1_0.WAD",
+    };
+
+    static const lump_rename_t sigil2_lumps [] = {
+        {"CREDIT",   "SG2CREDI"},
+        {"HELP1",    "SG2HELP1"},
+        {"TITLEPIC", "SG2TITLE"},
+        {"SIGILEND", "SGL2END"},
+        {"DEHACKED", "SG2_DEH"},
+        {"DEMO1",    "SG2DEMO1"},
+        {"DEMO2",    "SG2DEMO2"},
+        {"DEMO3",    "SG2DEMO3"},
+        {"DEMO4",    "SG2DEMO4"},
+        {"D_INTER",  "D_SG2INT"},
+        {"D_INTRO",  "D_SG2TIT"},
+    };
+
+    // [crispy] don't load Sigil II if another PWAD already provides E6M1
+    i = W_CheckNumForName("E6M1");
+    if (i != -1)
+    {
+        // [crispy] indicate that SIGIL_II_*.WAD is already loaded as a PWAD
+        if (!strncasecmp(W_WadNameForLump(lumpinfo[i]), "SIGIL_II", 8))
+        {
+            crispy->havesigil2 = (char *)-1;
+        }
+
+        return false;
+    }
+
+    // [crispy] don't load Sigil II if another PWAD already modifies the
+    // texture files
+    if (pwadtexture)
+    {
+        return false;
+    }
+
+    // [crispy] load Sigil II
+    for (i = 0; i < arrlen(sigil2_wads); i++)
+    {
+        crispy->havesigil2 = M_StringJoin(iwaddir, DIR_SEPARATOR_S,
+                                         sigil2_wads[i], NULL);
+
+        if (M_FileExists(crispy->havesigil2))
+        {
+            break;
+        }
+
+        free(crispy->havesigil2);
+        crispy->havesigil2 = D_FindWADByName(sigil2_wads[i]);
+
+        if (crispy->havesigil2)
+        {
+            break;
+        }
+    }
+
+    if (crispy->havesigil2 == NULL)
+    {
+        return false;
+    }
+
+    printf(" [Sigil II] adding %s\n", crispy->havesigil2);
+    W_MergeFile(crispy->havesigil2);
+    sigil2_basename = M_BaseName(crispy->havesigil2);
+
+    // [crispy] rename intrusive Sigil II graphics, demos and music lumps out
+    // of the way
+    for (i = 0; i < arrlen(sigil2_lumps); i++)
+    {
+        j = W_CheckNumForName(sigil2_lumps[i].name);
+
+        if (j != -1 && !strcasecmp(W_WadNameForLump(lumpinfo[j]), sigil2_basename))
+        {
+            memcpy(lumpinfo[j]->name, sigil2_lumps[i].new_name, 8);
+        }
+    }
+
+    // [crispy] load WAD and DEH files from autoload directories
+    if (!M_ParmExists("-noautoload"))
+    {
+        if ((autoload_dir = M_GetAutoloadDir(sigil2_basename, false)))
+        {
+            W_AutoLoadWADsRename(autoload_dir, sigil2_lumps, arrlen(sigil2_lumps));
+            DEH_AutoLoadPatches(autoload_dir);
+            free(autoload_dir);
+        }
+    }
+
+    return true;
+}
+
+// [crispy] auto-load Sigil and Sigil II if available
+void D_LoadSigilWads (void)
+{
+    int i, j;
+    boolean sigilloaded, sigil2loaded;
+    boolean pwadtexture = false;
+    char *iwaddir;
+
+    const char *const texture_file[] = {
+        "PNAMES",
+        "TEXTURE1",
+        "TEXTURE2",
+    };
+
+    // [crispy] don't load Sigils if another PWAD already modifies the texture
+    // files
+    for (i = 0; i < arrlen(texture_file); i++)
+    {
+        j = W_CheckNumForName(texture_file[i]);
+
+        if (j != -1 && !W_IsIWADLump(lumpinfo[j]))
+        {
+            pwadtexture = true;
+            break;
+        }
+    }
+
+    iwaddir = M_DirName(iwadfile);
+    sigilloaded = LoadSigilWad(iwaddir, pwadtexture);
+    sigil2loaded = LoadSigil2Wad(iwaddir, pwadtexture);
+
+    if (sigilloaded || sigil2loaded)
+    {
+        // [crispy] regenerate the hashtable
+        W_GenerateHashTable();
+    }
+
+    free(iwaddir);
 }
 
 // [crispy] check if NERVE.WAD is already loaded as a PWAD
@@ -221,10 +345,7 @@ static void CheckLoadNerve (void)
 	char *autoload_dir;
 	int i, j;
 
-	static const struct {
-		const char *name;
-		const char new_name[8];
-	} nerve_lumps [] = {
+	static const lump_rename_t nerve_lumps [] = {
 		{"TITLEPIC", "NERVEPIC"},
 		{"INTERPIC", "NERVEINT"},
 	};
@@ -293,7 +414,7 @@ static void CheckLoadNerve (void)
 	{
 		if ((autoload_dir = M_GetAutoloadDir(nerve_basename, false)))
 		{
-			W_AutoLoadWADs(autoload_dir);
+			W_AutoLoadWADsRename(autoload_dir, nerve_lumps, arrlen(nerve_lumps));
 			DEH_AutoLoadPatches(autoload_dir);
 			free(autoload_dir);
 		}
@@ -333,6 +454,11 @@ static boolean CheckMasterlevelsLoaded (void)
 	return false;
 }
 
+static const lump_rename_t master_lumps [] = {
+	{"TITLEPIC", "MASTRPIC"},
+	{"INTERPIC", "MASTRINT"},
+};
+
 // [crispy] auto-load the single MASTERLEVELS.WAD if available
 static boolean CheckLoadMasterlevels (void)
 {
@@ -402,7 +528,7 @@ static boolean CheckLoadMasterlevels (void)
 	{
 		if ((autoload_dir = M_GetAutoloadDir(master_basename, false)))
 		{
-			W_AutoLoadWADs(autoload_dir);
+			W_AutoLoadWADsRename(autoload_dir, master_lumps, arrlen(master_lumps));
 			DEH_AutoLoadPatches(autoload_dir);
 			free(autoload_dir);
 		}
@@ -504,6 +630,7 @@ static void LoadMasterlevelsWads (void)
 {
 	int i, j;
 	char lumpname[9];
+	char *autoload_dir;
 
 	const char *const sky_lumps[] = {
 		"RSKY1",
@@ -547,6 +674,17 @@ static void LoadMasterlevelsWads (void)
 	// [crispy] indicate this is not the single MASTERLEVELS.WAD
 	crispy->havemaster = (char *)-1;
 
+	// [crispy] autoload from MASTERLEVELS.WAD autoload directory
+	if (!M_ParmExists("-noautoload"))
+	{
+		if ((autoload_dir = M_GetAutoloadDir("MASTERLEVELS.WAD", false)))
+		{
+			W_AutoLoadWADsRename(autoload_dir, master_lumps, arrlen(master_lumps));
+			DEH_AutoLoadPatches(autoload_dir);
+			free(autoload_dir);
+		}
+	}
+
 	// [crispy] regenerate the hashtable
 	W_GenerateHashTable();
 	return;
