@@ -76,6 +76,7 @@ static textscreen_t textscreens[] =
     { doom,      3, 8,  "MFLR8_4",   E3TEXT},
     { doom,      4, 8,  "MFLR8_3",   E4TEXT},
     { doom,      5, 8,  "FLOOR7_2",  E5TEXT}, // [crispy] Sigil
+    { doom,      6, 8,  "FLOOR7_2",  E6TEXT}, // [crispy] Sigil II
 
     { doom2,     1, 6,  "SLIME16",   C1TEXT},
     { doom2,     1, 11, "RROCK14",   C2TEXT},
@@ -272,7 +273,7 @@ void F_TextWrite (void)
     byte*	src;
     pixel_t*	dest;
     
-    int		x,y,w;
+    int		w;
     signed int	count;
     char *ch; // [crispy] un-const
     int		c;
@@ -283,26 +284,8 @@ void F_TextWrite (void)
     src = W_CacheLumpName ( finaleflat , PU_CACHE);
     dest = I_VideoBuffer;
 	
-    for (y=0 ; y<SCREENHEIGHT ; y++)
-    {
-#ifndef CRISPY_TRUECOLOR
-	for (x=0 ; x<SCREENWIDTH/64 ; x++)
-	{
-	    memcpy (dest, src+((y&63)<<6), 64);
-	    dest += 64;
-	}
-	if (SCREENWIDTH&63)
-	{
-	    memcpy (dest, src+((y&63)<<6), SCREENWIDTH&63);
-	    dest += (SCREENWIDTH&63);
-	}
-#else
-	for (x=0 ; x<SCREENWIDTH ; x++)
-	{
-		*dest++ = colormaps[src[((y&63)<<6) + (x&63)]];
-	}
-#endif
-    }
+    // [crispy] use unified flat filling function
+    V_FillFlat(0, SCREENHEIGHT, 0, SCREENWIDTH, src, dest);
 
     V_MarkRect (0, 0, SCREENWIDTH, SCREENHEIGHT);
     
@@ -327,7 +310,7 @@ void F_TextWrite (void)
 	}
 		
 	c = toupper(c) - HU_FONTSTART;
-	if (c < 0 || c> HU_FONTSIZE)
+	if (c < 0 || c >= HU_FONTSIZE)
 	{
 	    cx += 4;
 	    continue;
@@ -794,7 +777,7 @@ void F_CastPrint (const char *text)
 	if (!c)
 	    break;
 	c = toupper(c) - HU_FONTSTART;
-	if (c < 0 || c> HU_FONTSIZE)
+	if (c < 0 || c >= HU_FONTSIZE)
 	{
 	    width += 4;
 	    continue;
@@ -813,7 +796,7 @@ void F_CastPrint (const char *text)
 	if (!c)
 	    break;
 	c = toupper(c) - HU_FONTSTART;
-	if (c < 0 || c> HU_FONTSIZE)
+	if (c < 0 || c >= HU_FONTSIZE)
 	{
 	    cx += 4;
 	    continue;
@@ -896,7 +879,7 @@ F_DrawPatchCol
 #ifndef CRISPY_TRUECOLOR
 	    *dest = source[srccol >> FRACBITS];
 #else
-	    *dest = colormaps[source[srccol >> FRACBITS]];
+	    *dest = pal_color[source[srccol >> FRACBITS]];
 #endif
 	    srccol += dyi;
 	    dest += SCREENWIDTH;
@@ -1036,6 +1019,19 @@ static void F_ArtScreenDrawer(void)
                     return;
                 }
                 break;
+            // [crispy] Sigil II
+            case 6:
+                lumpname = "SGL2END";
+                if (W_CheckNumForName(DEH_String(lumpname)) == -1)
+                {
+                    lumpname = "SIGILEND";
+
+                    if (W_CheckNumForName(DEH_String(lumpname)) == -1)
+                    {
+                        return;
+                    }
+                }
+                break;
             default:
                 return;
         }
