@@ -36,6 +36,7 @@
 #include "m_misc.h"
 #include "m_menu.h"
 #include "m_random.h"
+#include "i_joystick.h"
 #include "i_system.h"
 #include "i_timer.h"
 #include "i_input.h"
@@ -90,7 +91,6 @@ void	G_DoReborn (int playernum);
  
 void	G_DoLoadLevel (void); 
 void	G_DoNewGame (void); 
-void	G_DoPlayDemo (void); 
 void	G_DoCompleted (void); 
 void	G_DoVictory (void); 
 void	G_DoWorldDone (void); 
@@ -215,9 +215,20 @@ static boolean *mousebuttons = &mousearray[1];  // allow [-1]
 
 // mouse values are used once 
 int             mousex;
-int             mousex2;
 int             mousey;         
 
+// [crispy] for rounding error
+typedef struct carry_s
+{
+    double angle;
+    double pitch;
+    double side;
+    double vert;
+} carry_t;
+
+static carry_t prevcarry;
+static carry_t carry;
+
 static int      dclicktime;
 static boolean  dclickstate;
 static int      dclicks; 
@@ -237,6 +248,8 @@ static char     savename[256]; // [crispy] moved here, made static
 static int      savegameslot; 
 static char     savedescription[32]; 
  
+static ticcmd_t basecmd; // [crispy]
+
 #define	BODYQUESIZE	32
 
 mobj_t*		bodyque[BODYQUESIZE]; 
@@ -320,6 +333,12 @@ static int G_NextWeapon(int direction)
         }
     }
 
+    // The current weapon must be present in weapon_order_table
+    // otherwise something has gone terribly wrong
+    if (i >= arrlen(weapon_order_table)) {
+        I_Error("Internal error: weapon %d not present in weapon_order_table", weapon);
+    }
+
     // Switch weapon. Don't loop forever.
     start_i = i;
     do
@@ -340,6 +359,71 @@ boolean speedkeydown (void)
            (mousebspeed < MAX_MOUSE_BUTTONS && mousebuttons[mousebspeed]);
 }
 
+// [crispy] for carrying rounding error
+static int CarryError(double value, const double *prevcarry, double *carry)
+{
+    const double desired = value + *prevcarry;
+    const int actual = lround(desired);
+    *carry = desired - actual;
+
+    return actual;
+}
+
+static short CarryAngle(double angle)
+{
+    if (lowres_turn && abs(angle + prevcarry.angle) < 128)
+    {
+        carry.angle = angle + prevcarry.angle;
+        return 0;
+    }
+    else
+    {
+        return CarryError(angle, &prevcarry.angle, &carry.angle);
+    }
+}
+
+static short CarryPitch(double pitch)
+{
+    return CarryError(pitch, &prevcarry.pitch, &carry.pitch);
+}
+
+static int CarryMouseVert(double vert)
+{
+    return CarryError(vert, &prevcarry.vert, &carry.vert);
+}
+
+static int CarryMouseSide(double side)
+{
+    const double desired = side + prevcarry.side;
+    const int actual = lround(side * 0.5) * 2; // Even values only.
+    carry.side = desired - actual;
+    return actual;
+}
+
+static double CalcMouseAngle(int mousex)
+{
+    if (!mouseSensitivity)
+        return 0.0;
+
+    return (I_AccelerateMouse(mousex) * (mouseSensitivity + 5) * 8 / 10);
+}
+
+static double CalcMouseSide(int mousex)
+{
+    if (!mouseSensitivity_x2)
+        return 0.0;
+
+    return (I_AccelerateMouse(mousex) * (mouseSensitivity_x2 + 5) * 2 / 10);
+}
+
+static double CalcMouseVert(int mousey)
+{
+    if (!mouseSensitivity_y)
+        return 0.0;
+
+    return (I_AccelerateMouseY(mousey) * (mouseSensitivity_y + 5) / 10);
+}
+
 //
 // G_BuildTiccmd
 // Builds a ticcmd from all of the available inputs
@@ -354,13 +438,18 @@ void G_BuildTiccmd (ticcmd_t* cmd, int maketic)
     int		speed;
     int		tspeed; 
     int		lspeed;
+    int		angle = 0; // [crispy]
+    short	mousex_angleturn; // [crispy]
     int		forward;
     int		side;
     int		look;
     player_t *const player = &players[consoleplayer];
     static char playermessage[48];
 
-    memset(cmd, 0, sizeof(ticcmd_t));
+    // [crispy] For fast polling.
+    G_PrepTiccmd();
+    memcpy(cmd, &basecmd, sizeof(*cmd));
+    memset(&basecmd, 0, sizeof(ticcmd_t));
 
     cmd->consistancy = 
 	consistancy[consoleplayer][maketic%BACKUPTICS]; 
@@ -417,7 +506,7 @@ void G_BuildTiccmd (ticcmd_t* cmd, int maketic)
     // [crispy] add quick 180° reverse
     if (gamekeydown[key_reverse] || mousebuttons[mousebreverse])
     {
-        cmd->angleturn += ANG180 >> FRACBITS;
+        angle += ANG180 >> FRACBITS;
         gamekeydown[key_reverse] = false;
         mousebuttons[mousebreverse] = false;
     }
@@ -483,32 +572,54 @@ void G_BuildTiccmd (ticcmd_t* cmd, int maketic)
     // let movement keys cancel each other out
     if (strafe) 
     { 
-	if (gamekeydown[key_right] || mousebuttons[mousebturnright])
-	{
-	    // fprintf(stderr, "strafe right\n");
-	    side += sidemove[speed]; 
-	}
-	if (gamekeydown[key_left] || mousebuttons[mousebturnleft])
-	{
-	    //	fprintf(stderr, "strafe left\n");
-	    side -= sidemove[speed]; 
-	}
-	if (joyxmove > 0) 
-	    side += sidemove[speed]; 
-	if (joyxmove < 0) 
-	    side -= sidemove[speed]; 
- 
+        if (!cmd->angleturn)
+        {
+            if (gamekeydown[key_right] || mousebuttons[mousebturnright])
+            {
+                // fprintf(stderr, "strafe right\n");
+                side += sidemove[speed];
+            }
+            if (gamekeydown[key_left] || mousebuttons[mousebturnleft])
+            {
+                //	fprintf(stderr, "strafe left\n");
+                side -= sidemove[speed];
+            }
+            if (use_analog && joyxmove)
+            {
+                joyxmove = joyxmove * joystick_move_sensitivity / 10;
+                joyxmove = BETWEEN(-FRACUNIT, FRACUNIT, joyxmove);
+                side += FixedMul(sidemove[speed], joyxmove);
+            }
+            else if (joystick_move_sensitivity)
+            {
+                if (joyxmove > 0)
+                    side += sidemove[speed];
+                if (joyxmove < 0)
+                    side -= sidemove[speed];
+            }
+        }
     } 
     else 
     { 
 	if (gamekeydown[key_right] || mousebuttons[mousebturnright])
-	    cmd->angleturn -= angleturn[tspeed]; 
+	    angle -= angleturn[tspeed];
 	if (gamekeydown[key_left] || mousebuttons[mousebturnleft])
-	    cmd->angleturn += angleturn[tspeed]; 
-	if (joyxmove > 0) 
-	    cmd->angleturn -= angleturn[tspeed]; 
-	if (joyxmove < 0) 
-	    cmd->angleturn += angleturn[tspeed]; 
+	    angle += angleturn[tspeed];
+        if (use_analog && joyxmove)
+        {
+            // Cubic response curve allows for finer control when stick
+            // deflection is small.
+            joyxmove = FixedMul(FixedMul(joyxmove, joyxmove), joyxmove);
+            joyxmove = joyxmove * joystick_turn_sensitivity / 10;
+            angle -= FixedMul(angleturn[1], joyxmove);
+        }
+        else if (joystick_turn_sensitivity)
+        {
+            if (joyxmove > 0)
+                angle -= angleturn[tspeed];
+            if (joyxmove < 0)
+                angle += angleturn[tspeed];
+        }
     } 
  
     if (gamekeydown[key_up] || gamekeydown[key_alt_up]) // [crispy] add key_alt_*
@@ -522,44 +633,88 @@ void G_BuildTiccmd (ticcmd_t* cmd, int maketic)
 	forward -= forwardmove[speed]; 
     }
 
-    if (joyymove < 0) 
-        forward += forwardmove[speed]; 
-    if (joyymove > 0) 
-        forward -= forwardmove[speed]; 
+    if (use_analog && joyymove)
+    {
+        joyymove = joyymove * joystick_move_sensitivity / 10;
+        joyymove = BETWEEN(-FRACUNIT, FRACUNIT, joyymove);
+        forward -= FixedMul(forwardmove[speed], joyymove);
+    }
+    else if (joystick_move_sensitivity)
+    {
+        if (joyymove < 0)
+            forward += forwardmove[speed];
+        if (joyymove > 0)
+            forward -= forwardmove[speed];
+    }
 
     if (gamekeydown[key_strafeleft] || gamekeydown[key_alt_strafeleft] // [crispy] add key_alt_*
      || joybuttons[joybstrafeleft]
-     || mousebuttons[mousebstrafeleft]
-     || joystrafemove < 0)
+     || mousebuttons[mousebstrafeleft])
     {
         side -= sidemove[speed];
     }
 
     if (gamekeydown[key_straferight] || gamekeydown[key_alt_straferight] // [crispy] add key_alt_*
      || joybuttons[joybstraferight]
-     || mousebuttons[mousebstraferight]
-     || joystrafemove > 0)
+     || mousebuttons[mousebstraferight])
     {
         side += sidemove[speed]; 
     }
 
+    if (use_analog && joystrafemove)
+    {
+        joystrafemove = joystrafemove * joystick_move_sensitivity / 10;
+        joystrafemove = BETWEEN(-FRACUNIT, FRACUNIT, joystrafemove);
+        side += FixedMul(sidemove[speed], joystrafemove);
+    }
+    else if (joystick_move_sensitivity)
+    {
+        if (joystrafemove < 0)
+            side -= sidemove[speed];
+        if (joystrafemove > 0)
+            side += sidemove[speed];
+    }
+
     // [crispy] look up/down/center keys
     if (crispy->freelook)
     {
         static unsigned int kbdlookctrl = 0;
 
-        if (gamekeydown[key_lookup] || joylook < 0)
+        if (gamekeydown[key_lookup])
         {
             look = lspeed;
             kbdlookctrl += ticdup;
         }
         else
-        if (gamekeydown[key_lookdown] || joylook > 0)
+        if (gamekeydown[key_lookdown])
         {
             look = -lspeed;
             kbdlookctrl += ticdup;
         }
         else
+        if (joylook && joystick_look_sensitivity)
+        {
+            if (use_analog)
+            {
+                joylook = joylook * joystick_look_sensitivity / 10;
+                joylook = BETWEEN(-FRACUNIT, FRACUNIT, joylook);
+                look = -FixedMul(2, joylook);
+            }
+            else
+            {
+                if (joylook < 0)
+                {
+                    look = lspeed;
+                }
+
+                if (joylook > 0)
+                {
+                    look = -lspeed;
+                }
+            }
+            kbdlookctrl += ticdup;
+        }
+        else
         // [crispy] keyboard lookspring
         if (gamekeydown[key_lookcenter] || (crispy->freelook == FREELOOK_SPRING && kbdlookctrl))
         {
@@ -691,12 +846,13 @@ void G_BuildTiccmd (ticcmd_t* cmd, int maketic)
     if ((crispy->freelook && mousebuttons[mousebmouselook]) ||
          crispy->mouselook)
     {
-        cmd->lookdir = mouse_y_invert ? -mousey : mousey;
+        const double vert = CalcMouseVert(mousey);
+        cmd->lookdir += mouse_y_invert ? CarryPitch(-vert) : CarryPitch(vert);
     }
     else
     if (!novert)
     {
-    forward += mousey;
+    forward += CarryMouseVert(CalcMouseVert(mousey));
     }
 
     // [crispy] single click on mouse look button centers view
@@ -721,19 +877,27 @@ void G_BuildTiccmd (ticcmd_t* cmd, int maketic)
         }
     }
 
-    if (strafe) 
-	side += mousex2*2;
-    else 
-	cmd->angleturn -= mousex*0x8; 
+    if (strafe && !cmd->angleturn)
+	side += CarryMouseSide(CalcMouseSide(mousex));
 
-    if (mousex == 0)
+    mousex_angleturn = cmd->angleturn;
+
+    if (mousex_angleturn == 0)
     {
         // No movement in the previous frame
 
         testcontrols_mousespeed = 0;
     }
     
-    mousex = mousex2 = mousey = 0;
+    if (angle)
+    {
+        cmd->angleturn = CarryAngle(cmd->angleturn + angle);
+        localview.ticangleturn = crispy->fliplevels ?
+            (mousex_angleturn - cmd->angleturn) :
+            (cmd->angleturn - mousex_angleturn);
+    }
+
+    mousex = mousey = 0;
 	 
     if (forward > MAXPLMOVE) 
 	forward = MAXPLMOVE; 
@@ -747,6 +911,11 @@ void G_BuildTiccmd (ticcmd_t* cmd, int maketic)
     cmd->forwardmove += forward; 
     cmd->sidemove += side;
 
+    // [crispy]
+    localview.angle = 0;
+    localview.rawangle = 0.0;
+    prevcarry = carry;
+
     // [crispy] lookdir delta is stored in the lower 4 bits of the lookfly variable
     if (player->playerstate == PST_LIVE)
     {
@@ -758,7 +927,8 @@ void G_BuildTiccmd (ticcmd_t* cmd, int maketic)
     }
     
     // special buttons
-    if (sendpause) 
+    // [crispy] suppress pause when a new game is started
+    if (sendpause && gameaction != ga_newgame) 
     { 
 	sendpause = false; 
 	// [crispy] ignore un-pausing in menus during demo recording
@@ -776,6 +946,7 @@ void G_BuildTiccmd (ticcmd_t* cmd, int maketic)
 
     if (crispy->fliplevels)
     {
+	mousex_angleturn = -mousex_angleturn;
 	cmd->angleturn = -cmd->angleturn;
 	cmd->sidemove = -cmd->sidemove;
     }
@@ -784,20 +955,26 @@ void G_BuildTiccmd (ticcmd_t* cmd, int maketic)
 
     if (lowres_turn)
     {
-        static signed short carry = 0;
         signed short desired_angleturn;
 
-        desired_angleturn = cmd->angleturn + carry;
+        desired_angleturn = cmd->angleturn;
 
         // round angleturn to the nearest 256 unit boundary
         // for recording demos with single byte values for turn
 
         cmd->angleturn = (desired_angleturn + 128) & 0xff00;
 
+        if (angle)
+        {
+            localview.ticangleturn = cmd->angleturn - mousex_angleturn;
+        }
+
         // Carry forward the error from the reduced resolution to the
         // next tic, so that successive small movements can accumulate.
 
-        carry = desired_angleturn - cmd->angleturn;
+        prevcarry.angle += crispy->fliplevels ?
+                            cmd->angleturn - desired_angleturn :
+                            desired_angleturn - cmd->angleturn;
     }
 } 
  
@@ -915,7 +1092,11 @@ void G_DoLoadLevel (void)
 
     memset (gamekeydown, 0, sizeof(gamekeydown));
     joyxmove = joyymove = joystrafemove = joylook = 0;
-    mousex = mousex2 = mousey = 0;
+    mousex = mousey = 0;
+    memset(&localview, 0, sizeof(localview)); // [crispy]
+    memset(&carry, 0, sizeof(carry)); // [crispy]
+    memset(&prevcarry, 0, sizeof(prevcarry)); // [crispy]
+    memset(&basecmd, 0, sizeof(basecmd)); // [crispy]
     sendpause = sendsave = paused = false;
     memset(mousearray, 0, sizeof(mousearray));
     memset(joyarray, 0, sizeof(joyarray));
@@ -1048,6 +1229,7 @@ boolean G_Responder (event_t* ev)
 	    if (!menuactive && crispy->soundfix)
 		S_StartSoundOptional(NULL, sfx_mnuopn, sfx_swtchn); // [NS] Optional menu sounds.
 	    M_StartControlPanel (); 
+	    joywait = I_GetTime() + 5;
 	    return true; 
 	} 
 	return false; 
@@ -1119,18 +1301,8 @@ boolean G_Responder (event_t* ev)
 		 
       case ev_mouse: 
         SetMouseButtons(ev->data1);
-	if (mouseSensitivity)
-	mousex = ev->data2*(mouseSensitivity+5)/10; 
-	else
-	    mousex = 0; // [crispy] disable entirely
-	if (mouseSensitivity_x2)
-	mousex2 = ev->data2*(mouseSensitivity_x2+5)/10; // [crispy] separate sensitivity for strafe
-	else
-	    mousex2 = 0; // [crispy] disable entirely
-	if (mouseSensitivity_y)
-	mousey = ev->data3*(mouseSensitivity_y+5)/10; // [crispy] separate sensitivity for y-axis
-	else
-	    mousey = 0; // [crispy] disable entirely
+        mousex += ev->data2;
+        mousey += ev->data3;
 	return true;    // eat events 
  
       case ev_joystick: 
@@ -1148,7 +1320,42 @@ boolean G_Responder (event_t* ev)
     return false; 
 } 
  
+// [crispy] For fast polling.
+void G_FastResponder (void)
+{
+    if (newfastmouse)
+    {
+        mousex += fastmouse.data2;
+        mousey += fastmouse.data3;
+
+        newfastmouse = false;
+    }
+}
  
+// [crispy]
+void G_PrepTiccmd (void)
+{
+    const boolean strafe = gamekeydown[key_strafe] ||
+        mousebuttons[mousebstrafe] || joybuttons[joybstrafe];
+
+    if (mousex && !strafe)
+    {
+        localview.rawangle -= CalcMouseAngle(mousex);
+        basecmd.angleturn = CarryAngle(localview.rawangle);
+        localview.angle = crispy->fliplevels ?
+            -(basecmd.angleturn << 16) : (basecmd.angleturn << 16);
+        mousex = 0;
+    }
+
+    if (mousey && crispy->mouselook)
+    {
+        const double vert = CalcMouseVert(mousey);
+        basecmd.lookdir += mouse_y_invert ?
+                            CarryPitch(-vert): CarryPitch(vert);
+        mousey = 0;
+    }
+}
+
 // [crispy] re-read game parameters from command line
 static void G_ReadGameParms (void)
 {
@@ -1690,7 +1897,7 @@ void G_ScreenShot (void)
 
 
 // DOOM Par Times
-static const int pars[6][10] =
+static const int pars[7][10] =
 { 
     {0}, 
     {0,30,75,120,90,165,180,180,30,165}, 
@@ -1700,6 +1907,8 @@ static const int pars[6][10] =
    ,{0,165,255,135,150,180,390,135,360,180}
     // [crispy] Episode 5 par times from Sigil v1.21
    ,{0,90,150,360,420,780,420,780,300,660}
+    // [crispy] Episode 6 par times from Sigil II v1.0
+   ,{0,480,300,240,420,510,840,960,390,450}
 }; 
 
 // DOOM II Par Times
@@ -1968,6 +2177,7 @@ void G_DoCompleted (void)
 	    switch (gameepisode) 
 	    { 
 	      case 1: 
+	      case 6:
 		wminfo.next = 3; 
 		break; 
 	      case 2: 
@@ -2029,7 +2239,9 @@ void G_DoCompleted (void)
         // [crispy] single player par times for episode 4
         (gameepisode == 4 && crispy->singleplayer) ||
         // [crispy] par times for Sigil
-        gameepisode == 5)
+        gameepisode == 5 ||
+        // [crispy] par times for Sigil II
+        gameepisode == 6)
     {
         // [crispy] support [PARS] sections in BEX files
         if (bex_pars[gameepisode][gamemap])
@@ -2538,6 +2750,14 @@ G_InitNew
 
     M_ClearRandom ();
 
+    // [crispy] Spider Mastermind gets increased health in Sigil II. Normally
+    // the Sigil II DEH handles this, but we don't load the DEH if the WAD gets
+    // sideloaded.
+    if (crispy->havesigil2 && crispy->havesigil2 != (char *)-1)
+    {
+        mobjinfo[MT_SPIDER].spawnhealth = (episode == 6) ? 9000 : 3000;
+    }
+
     if (skill == sk_nightmare || respawnparm )
 	respawnmonsters = true;
     else
@@ -2629,6 +2849,13 @@ G_InitNew
                 skytexturename = "SKY3";
             }
             break;
+          case 6:        // [crispy] Sigil II
+            skytexturename = "SKY6_ZD";
+            if (R_CheckTextureNumForName(DEH_String(skytexturename)) == -1)
+            {
+                skytexturename = "SKY3";
+            }
+            break;
         }
         skytexturename = DEH_String(skytexturename);
         skytexture = R_TextureNumForName(skytexturename);
@@ -3131,24 +3358,19 @@ void G_TimeDemo (char* name)
 } 
  
 #define DEMO_FOOTER_SEPARATOR "\n"
+#define NUM_DEMO_FOOTER_LUMPS 4
 
-static void G_AddDemoFooter(void)
+static size_t WriteCmdLineLump(MEMFILE *stream)
 {
     int i;
-    size_t len = 0;
+    long pos;
     char *tmp, **filenames;
 
-    MEMFILE *stream = mem_fopen_write();
-
     filenames = W_GetWADFileNames();
 
-    if (!filenames)
-    {
-        return;
-    }
+    pos = mem_ftell(stream);
 
-    tmp = M_StringJoin(PACKAGE_STRING, DEMO_FOOTER_SEPARATOR,
-            "-iwad \"", M_BaseName(filenames[0]), "\"", NULL);
+    tmp = M_StringJoin("-iwad \"", M_BaseName(filenames[0]), "\"", NULL);
     mem_fputs(tmp, stream);
     free(tmp);
 
@@ -3178,26 +3400,93 @@ static void G_AddDemoFooter(void)
         }
     }
 
-    tmp = M_StringJoin(" -gameversion ", D_GetGameVersionCmd(), NULL);
-    mem_fputs(tmp, stream);
-    free(tmp);
+    switch (gameversion)
+    {
+        case exe_doom_1_2:
+            mem_fputs(" -complevel 0", stream);
+            break;
+        case exe_doom_1_666:
+            mem_fputs(" -complevel 1", stream);
+            break;
+        case exe_doom_1_9:
+            mem_fputs(" -complevel 2", stream);
+            break;
+        case exe_ultimate:
+            mem_fputs(" -complevel 3", stream);
+            break;
+        case exe_final:
+            mem_fputs(" -complevel 4", stream);
+            break;
+        default:
+            break;
+    }
 
     if (M_CheckParm("-solo-net"))
     {
         mem_fputs(" -solo-net", stream);
     }
 
+    return mem_ftell(stream) - pos;
+}
+
+static void WriteFileInfo(const char *name, size_t size, MEMFILE *stream)
+{
+    filelump_t fileinfo = { 0 };
+    static long filepos = sizeof(wadinfo_t);
+
+    fileinfo.filepos = LONG(filepos);
+    fileinfo.size = LONG(size);
+
+    if (name)
+    {
+        size_t len = strnlen(name, 8);
+        if (len < 8)
+        {
+            len++;
+        }
+        memcpy(fileinfo.name, name, len);
+    }
+
+    mem_fwrite(&fileinfo, 1, sizeof(fileinfo), stream);
+
+    filepos += size;
+}
+
+static void G_AddDemoFooter(void)
+{
+    byte *data;
+    size_t size;
+
+    MEMFILE *stream = mem_fopen_write();
+
+    wadinfo_t header = { "PWAD" };
+    header.numlumps = LONG(NUM_DEMO_FOOTER_LUMPS);
+    mem_fwrite(&header, 1, sizeof(header), stream);
+
+    mem_fputs(PACKAGE_STRING, stream);
+    mem_fputs(DEMO_FOOTER_SEPARATOR, stream);
+    size = WriteCmdLineLump(stream);
     mem_fputs(DEMO_FOOTER_SEPARATOR, stream);
 
-    mem_get_buf(stream, (void **)&tmp, &len);
+    header.infotableofs = LONG(mem_ftell(stream));
+    mem_fseek(stream, 0, MEM_SEEK_SET);
+    mem_fwrite(&header, 1, sizeof(header), stream);
+    mem_fseek(stream, 0, MEM_SEEK_END);
 
-    while (demo_p > demoend - len)
+    WriteFileInfo("PORTNAME", strlen(PACKAGE_STRING), stream);
+    WriteFileInfo(NULL, strlen(DEMO_FOOTER_SEPARATOR), stream);
+    WriteFileInfo("CMDLINE", size, stream);
+    WriteFileInfo(NULL, strlen(DEMO_FOOTER_SEPARATOR), stream);
+
+    mem_get_buf(stream, (void **)&data, &size);
+
+    while (demo_p > demoend - size)
     {
         IncreaseDemoBuffer();
     }
 
-    memcpy(demo_p, tmp, len);
-    demo_p += len;
+    memcpy(demo_p, data, size);
+    demo_p += size;
 
     mem_fclose(stream);
 }
@@ -3312,5 +3601,23 @@ boolean G_CheckDemoStatus (void)
     return false; 
 } 
  
+//
+// G_DemoGotoNextLevel
+// [crispy] fast forward to next level while demo playback
+//
+
+boolean demo_gotonextlvl;
+
+void G_DemoGotoNextLevel (boolean start)
+{
+    // disable screen rendering while fast forwarding
+    nodrawers = start;
+
+    // switch to fast tics running mode if not in -timedemo
+    if (!timingdemo)
+    {
+        singletics = start;
+    }
+} 
  
  
