@@ -36,7 +36,7 @@
 #include "m_menu.h"
 #include "w_wad.h"
 #include "m_argv.h" // [crispy] M_ParmExists()
-#include "st_stuff.h" // [crispy] ST_HEIGHT
+#include "st_stuff.h" // [crispy] ST_HEIGHT, ST_WIDESCREENDELTA
 #include "p_setup.h" // maplumpinfo
 
 #include "s_sound.h"
@@ -47,7 +47,7 @@
 #include "dstrings.h"
 #include "sounds.h"
 
-#include "r_state.h" // [crispy] colormaps
+#include "r_state.h" // [crispy] pal_color
 #include "v_video.h" // [crispy] V_DrawPatch() et al.
 #include "v_trans.h" // [crispy] colored kills/items/secret/etc. messages
 
@@ -175,6 +175,17 @@ const char *mapnames[] =	// DOOM shareware/registered/retail (Ultimate) names.
     HUSTR_E5M8,
     HUSTR_E5M9,
 
+    // [crispy] Sigil II
+    HUSTR_E6M1,
+    HUSTR_E6M2,
+    HUSTR_E6M3,
+    HUSTR_E6M4,
+    HUSTR_E6M5,
+    HUSTR_E6M6,
+    HUSTR_E6M7,
+    HUSTR_E6M8,
+    HUSTR_E6M9,
+
     "NEWLEVEL",
     "NEWLEVEL",
     "NEWLEVEL",
@@ -771,12 +782,12 @@ void HU_DemoProgressBar (void)
 //  V_DrawHorizLine(0, SCREENHEIGHT - 2, 1, 4); // [crispy] white start
 //  V_DrawHorizLine(i - 1, SCREENHEIGHT - 2, 1, 4); // [crispy] white end
 #else
-//  V_DrawHorizLine(0, SCREENHEIGHT - 3, i, colormaps[4]); // [crispy] white
-    V_DrawHorizLine(0, SCREENHEIGHT - 2, i, colormaps[0]); // [crispy] black
-    V_DrawHorizLine(0, SCREENHEIGHT - 1, i, colormaps[4]); // [crispy] white
+//  V_DrawHorizLine(0, SCREENHEIGHT - 3, i, pal_color[4]); // [crispy] white
+    V_DrawHorizLine(0, SCREENHEIGHT - 2, i, pal_color[0]); // [crispy] black
+    V_DrawHorizLine(0, SCREENHEIGHT - 1, i, pal_color[4]); // [crispy] white
 
-//  V_DrawHorizLine(0, SCREENHEIGHT - 2, 1, colormaps[4]); // [crispy] white start
-//  V_DrawHorizLine(i - 1, SCREENHEIGHT - 2, 1, colormaps[4]); // [crispy] white end
+//  V_DrawHorizLine(0, SCREENHEIGHT - 2, 1, pal_color[4]); // [crispy] white start
+//  V_DrawHorizLine(i - 1, SCREENHEIGHT - 2, 1, pal_color[4]); // [crispy] white end
 #endif
 }
 
@@ -1087,6 +1098,8 @@ void HU_Ticker(void)
     {
 	crispy_statsline_func_t crispy_statsline = crispy_statslines[crispy->statsformat];
 
+	w_kills.x = - ST_WIDESCREENDELTA;
+
 	w_kills.y = HU_TITLEY;
 
 	crispy_statsline(str, sizeof(str), "K ", plr->killcount, totalkills, extrakills);
@@ -1108,8 +1121,11 @@ void HU_Ticker(void)
     else
     if ((crispy->automapstats & WIDGETS_ALWAYS) || (automapactive && crispy->automapstats == WIDGETS_AUTOMAP))
     {
+
 	crispy_statsline_func_t crispy_statsline = crispy_statslines[crispy->statsformat];
 
+	w_kills.x = HU_TITLEX; // to handle switching from Status bar to Always and Automap kills line options
+
 	crispy_statsline(str, sizeof(str), kills, plr->killcount, totalkills, extrakills);
 	HUlib_clearTextLine(&w_kills);
 	s = str;
