@@ -81,7 +81,18 @@ musicinfo_t S_music[] =
     MUSIC("e5m7"),
     MUSIC("e5m8"),
     MUSIC("e5m9"),
+    // [crispy] Sigil II
+    MUSIC("e6m1"),
+    MUSIC("e6m2"),
+    MUSIC("e6m3"),
+    MUSIC("e6m4"),
+    MUSIC("e6m5"),
+    MUSIC("e6m6"),
+    MUSIC("e6m7"),
+    MUSIC("e6m8"),
+    MUSIC("e6m9"),
     MUSIC("sigint"),
+    MUSIC("sg2int"),
     MUSIC("inter"),
     MUSIC("intro"),
     MUSIC("bunny"),
@@ -122,6 +133,16 @@ musicinfo_t S_music[] =
     MUSIC("read_m"),
     MUSIC("dm2ttl"),
     MUSIC("dm2int"),
+    // [crispy] NRFTL
+    MUSIC("nrftl1"),
+    MUSIC("nrftl2"),
+    MUSIC("nrftl3"),
+    MUSIC("nrftl4"),
+    MUSIC("nrftl5"),
+    MUSIC("nrftl6"),
+    MUSIC("nrftl7"),
+    MUSIC("nrftl8"),
+    MUSIC("nrftl9"),
     MUSIC(NULL),
     MUSIC("musinfo")
 };
