@@ -82,7 +82,18 @@ typedef enum
     mus_e5m7,
     mus_e5m8,
     mus_e5m9,
+    // [crispy] Sigil II
+    mus_e6m1,
+    mus_e6m2,
+    mus_e6m3,
+    mus_e6m4,
+    mus_e6m5,
+    mus_e6m6,
+    mus_e6m7,
+    mus_e6m8,
+    mus_e6m9,
     mus_sigint,
+    mus_sg2int,
     mus_inter,
     mus_intro,
     mus_bunny,
@@ -123,6 +134,16 @@ typedef enum
     mus_read_m,
     mus_dm2ttl,
     mus_dm2int,
+    // [crispy] NRFTL
+    mus_nrftl1,
+    mus_nrftl2,
+    mus_nrftl3,
+    mus_nrftl4,
+    mus_nrftl5,
+    mus_nrftl6,
+    mus_nrftl7,
+    mus_nrftl8,
+    mus_nrftl9,
     NUMMUSIC,
     mus_musinfo
 } musicenum_t;
