@@ -1821,7 +1821,7 @@ void A_GauntletAttack(mobj_t *actor, player_t *player, pspdef_t *psp)
                             linetarget->x, linetarget->y);
     if (angle - player->mo->angle > ANG180)
     {
-        if (angle - player->mo->angle < -ANG90 / 20)
+        if ((signed int) (angle - player->mo->angle) < -ANG90 / 20)
             player->mo->angle = angle + ANG90 / 21;
         else
             player->mo->angle -= ANG90 / 20;
