@@ -219,10 +219,6 @@ struct
 
 mobj_t LavaInflictor;
 
-// [AM] Fractional part of the current tic, in the half-open
-//      range of [0.0, 1.0).  Used for interpolation.
-extern fixed_t          fractionaltic;
-
 //----------------------------------------------------------------------------
 //
 // PROC P_InitLava
@@ -896,6 +892,7 @@ void P_ShootSpecialLine(mobj_t * thing, line_t * line)
 void P_PlayerInSpecialSector(player_t * player)
 {
     sector_t *sector;
+    static sector_t *error; // [crispy] for sectors with unknown special
     static int pushTab[5] = {
         2048 * 5,
         2048 * 10,
@@ -1018,8 +1015,14 @@ void P_PlayerInSpecialSector(player_t * player)
             break;
 
         default:
-            I_Error("P_PlayerInSpecialSector: "
-                    "unknown special %i", sector->special);
+            // [crispy] ignore unknown special sectors
+            if (error != sector)
+            {
+                error = sector;
+                printf("P_PlayerInSpecialSector: "
+                       "unknown special %i\n", sector->special);
+            }
+            break;
     }
 }
 
@@ -1083,7 +1086,7 @@ void P_UpdateSpecials(void)
         }
     }
     // Handle buttons
-    for (i = 0; i < MAXBUTTONS; i++)
+    for (i = 0; i < maxbuttons; i++)
     {
         if (buttonlist[i].btimer)
         {
@@ -1285,6 +1288,7 @@ void P_SpawnSpecials(void)
     //
     numlinespecials = 0;
     for (i = 0; i < numlines; i++)
+    {
         switch (lines[i].special)
         {
             case 48:           // Effect_Scroll_Left
@@ -1292,7 +1296,22 @@ void P_SpawnSpecials(void)
                 linespeciallist[numlinespecials] = &lines[i];
                 numlinespecials++;
                 break;
+            // [crispy] add support for MBF sky transfers
+            case 271:
+            case 272:
+              {
+                int secnum;
+                for (secnum = 0; secnum < numsectors; secnum++)
+                  {
+                    if (sectors[secnum].tag == lines[i].tag)
+                        {
+                            sectors[secnum].sky = i | PL_SKYFLAT;
+                        }
+                  }
+              }
+             break;
         }
+    }
 
     //
     //      Init other misc stuff
@@ -1301,7 +1320,7 @@ void P_SpawnSpecials(void)
         activeceilings[i] = NULL;
     for (i = 0; i < MAXPLATS; i++)
         activeplats[i] = NULL;
-    for (i = 0; i < MAXBUTTONS; i++)
+    for (i = 0; i < maxbuttons; i++)
         memset(&buttonlist[i], 0, sizeof(button_t));
 }
 
