@@ -33,7 +33,6 @@
 // PRIVATE FUNCTION PROTOTYPES ---------------------------------------------
 
 // EXTERNAL DATA DECLARATIONS ----------------------------------------------
-extern fixed_t FloatBobOffsets[64];
 
 // PUBLIC DATA DEFINITIONS -------------------------------------------------
 int orbitTableX[256] = {
@@ -144,7 +143,7 @@ void A_DripBlood(mobj_t *actor)
 //
 //============================================================================
 
-void A_PotteryExplode(mobj_t * actor)
+void A_PotteryExplode(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     mobj_t *mo = NULL;
     int i;
@@ -177,7 +176,7 @@ void A_PotteryExplode(mobj_t * actor)
 //
 //============================================================================
 
-void A_PotteryChooseBit(mobj_t * actor)
+void A_PotteryChooseBit(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     P_SetMobjState(actor, actor->info->deathstate + (P_Random() % 5) + 1);
     actor->tics = 256 + (P_Random() << 1);
@@ -189,7 +188,7 @@ void A_PotteryChooseBit(mobj_t * actor)
 //
 //============================================================================
 
-void A_PotteryCheck(mobj_t * actor)
+void A_PotteryCheck(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     int i;
     mobj_t *pmo;
@@ -234,7 +233,7 @@ void A_PotteryCheck(mobj_t * actor)
 //
 //============================================================================
 
-void A_CorpseBloodDrip(mobj_t * actor)
+void A_CorpseBloodDrip(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (P_Random() > 128)
     {
@@ -250,7 +249,7 @@ void A_CorpseBloodDrip(mobj_t * actor)
 //
 //============================================================================
 
-void A_CorpseExplode(mobj_t * actor)
+void A_CorpseExplode(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     mobj_t *mo;
     int i;
@@ -282,7 +281,7 @@ void A_CorpseExplode(mobj_t * actor)
 //
 //============================================================================
 
-void A_LeafSpawn(mobj_t * actor)
+void A_LeafSpawn(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     mobj_t *mo;
     int i;
@@ -313,7 +312,7 @@ void A_LeafSpawn(mobj_t * actor)
 //
 //============================================================================
 
-void A_LeafThrust(mobj_t * actor)
+void A_LeafThrust(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (P_Random() > 96)
     {
@@ -328,7 +327,7 @@ void A_LeafThrust(mobj_t * actor)
 //
 //============================================================================
 
-void A_LeafCheck(mobj_t * actor)
+void A_LeafCheck(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     actor->special1.i++;
     if (actor->special1.i >= 20)
@@ -404,7 +403,7 @@ void GenerateOrbitTable(void)
 //              target          pointer to center mobj
 //              args[0]         angle of ball
 
-void A_BridgeOrbit(mobj_t * actor)
+void A_BridgeOrbit(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (actor->target->special1.i)
     {
@@ -417,7 +416,7 @@ void A_BridgeOrbit(mobj_t * actor)
 }
 
 
-void A_BridgeInit(mobj_t * actor)
+void A_BridgeInit(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     byte startangle;
     mobj_t *ball1, *ball2, *ball3;
@@ -444,9 +443,9 @@ void A_BridgeInit(mobj_t * actor)
     ball3->args[0] = (startangle + 170) & 255;
     ball3->target = actor;
 
-    A_BridgeOrbit(ball1);
-    A_BridgeOrbit(ball2);
-    A_BridgeOrbit(ball3);
+    A_BridgeOrbit(ball1, NULL, NULL);
+    A_BridgeOrbit(ball2, NULL, NULL);
+    A_BridgeOrbit(ball3, NULL, NULL);
 }
 
 void A_BridgeRemove(mobj_t * actor)
@@ -489,7 +488,7 @@ void A_GhostOff(mobj_t *actor)
 //
 //==========================================================================
 
-void A_HideThing(mobj_t * actor)
+void A_HideThing(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     actor->flags2 |= MF2_DONTDRAW;
 }
@@ -500,7 +499,7 @@ void A_HideThing(mobj_t * actor)
 //
 //==========================================================================
 
-void A_UnHideThing(mobj_t * actor)
+void A_UnHideThing(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     actor->flags2 &= ~MF2_DONTDRAW;
 }
@@ -511,7 +510,7 @@ void A_UnHideThing(mobj_t * actor)
 //
 //==========================================================================
 
-void A_SetShootable(mobj_t * actor)
+void A_SetShootable(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     actor->flags2 &= ~MF2_NONSHOOTABLE;
     actor->flags |= MF_SHOOTABLE;
@@ -523,7 +522,7 @@ void A_SetShootable(mobj_t * actor)
 //
 //==========================================================================
 
-void A_UnSetShootable(mobj_t * actor)
+void A_UnSetShootable(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     actor->flags2 |= MF2_NONSHOOTABLE;
     actor->flags &= ~MF_SHOOTABLE;
@@ -535,7 +534,7 @@ void A_UnSetShootable(mobj_t * actor)
 //
 //==========================================================================
 
-void A_SetAltShadow(mobj_t * actor)
+void A_SetAltShadow(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     actor->flags &= ~MF_SHADOW;
     actor->flags |= MF_ALTSHADOW;
@@ -566,7 +565,7 @@ void A_UnSetAltShadow(mobj_t *actor)
 //
 //==========================================================================
 
-void A_ContMobjSound(mobj_t * actor)
+void A_ContMobjSound(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     switch (actor->type)
     {
@@ -590,7 +589,7 @@ void A_ContMobjSound(mobj_t * actor)
 //
 //==========================================================================
 
-void A_ESound(mobj_t * mo)
+void A_ESound(mobj_t *mo, player_t *player, pspdef_t *psp)
 {
     int sound;
 
@@ -613,7 +612,7 @@ void A_ESound(mobj_t * mo)
 //==========================================================================
 
 
-void A_Summon(mobj_t * actor)
+void A_Summon(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     mobj_t *mo;
     mobj_t *master;
@@ -668,7 +667,7 @@ void A_Summon(mobj_t * actor)
 //
 //==========================================================================
 
-void A_FogSpawn(mobj_t * actor)
+void A_FogSpawn(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     mobj_t *mo = NULL;
     angle_t delta;
@@ -709,7 +708,7 @@ void A_FogSpawn(mobj_t * actor)
 }
 
 
-void A_FogMove(mobj_t * actor)
+void A_FogMove(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     int speed = actor->args[0] << FRACBITS;
     angle_t angle;
@@ -742,7 +741,7 @@ void A_FogMove(mobj_t * actor)
 //
 //===========================================================================
 
-void A_PoisonBagInit(mobj_t * actor)
+void A_PoisonBagInit(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     mobj_t *mo;
 
@@ -767,7 +766,7 @@ void A_PoisonBagInit(mobj_t * actor)
 //
 //===========================================================================
 
-void A_PoisonBagCheck(mobj_t * actor)
+void A_PoisonBagCheck(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (!--actor->special1.i)
     {
@@ -785,13 +784,11 @@ void A_PoisonBagCheck(mobj_t * actor)
 //
 //===========================================================================
 
-void A_PoisonBagDamage(mobj_t * actor)
+void A_PoisonBagDamage(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     int bobIndex;
 
-    extern void A_Explode(mobj_t * actor);
-
-    A_Explode(actor);
+    A_Explode(actor, player, psp);
 
     bobIndex = actor->special2.i;
     actor->z += FloatBobOffsets[bobIndex] >> 4;
@@ -804,7 +801,7 @@ void A_PoisonBagDamage(mobj_t * actor)
 //
 //===========================================================================
 
-void A_PoisonShroom(mobj_t * actor)
+void A_PoisonShroom(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     actor->tics = 128 + (P_Random() << 1);
 }
@@ -815,7 +812,7 @@ void A_PoisonShroom(mobj_t * actor)
 //
 //===========================================================================
 
-void A_CheckThrowBomb(mobj_t * actor)
+void A_CheckThrowBomb(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (abs(actor->momx) < 1.5 * FRACUNIT && abs(actor->momy) < 1.5 * FRACUNIT
         && actor->momz < 2 * FRACUNIT
@@ -888,7 +885,7 @@ boolean A_LocalQuake(byte * args, mobj_t * actor)
 //===========================================================================
 int localQuakeHappening[MAXPLAYERS];
 
-void A_Quake(mobj_t * actor)
+void A_Quake(mobj_t *actor, player_t *player_param, pspdef_t *psp)
 {
     angle_t an;
     player_t *player;
@@ -947,7 +944,7 @@ void A_Quake(mobj_t * actor)
 
 #define TELEPORT_LIFE 1
 
-void A_TeloSpawnA(mobj_t * actor)
+void A_TeloSpawnA(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     mobj_t *mo;
 
@@ -963,7 +960,7 @@ void A_TeloSpawnA(mobj_t * actor)
     }
 }
 
-void A_TeloSpawnB(mobj_t * actor)
+void A_TeloSpawnB(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     mobj_t *mo;
 
@@ -979,7 +976,7 @@ void A_TeloSpawnB(mobj_t * actor)
     }
 }
 
-void A_TeloSpawnC(mobj_t * actor)
+void A_TeloSpawnC(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     mobj_t *mo;
 
@@ -995,7 +992,7 @@ void A_TeloSpawnC(mobj_t * actor)
     }
 }
 
-void A_TeloSpawnD(mobj_t * actor)
+void A_TeloSpawnD(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     mobj_t *mo;
 
@@ -1011,7 +1008,7 @@ void A_TeloSpawnD(mobj_t * actor)
     }
 }
 
-void A_CheckTeleRing(mobj_t * actor)
+void A_CheckTeleRing(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (actor->special1.i-- <= 0)
     {
@@ -1079,7 +1076,7 @@ void P_SpawnDirt(mobj_t * actor, fixed_t radius)
 //              args[1]         0 = normal,   1 = bloody
 //===========================================================================
 
-void A_ThrustInitUp(mobj_t * actor)
+void A_ThrustInitUp(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     actor->special2.i = 5;        // Raise speed
     actor->args[0] = 1;         // Mark as up
@@ -1089,7 +1086,7 @@ void A_ThrustInitUp(mobj_t * actor)
     actor->special1.m = NULL;
 }
 
-void A_ThrustInitDn(mobj_t * actor)
+void A_ThrustInitDn(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     mobj_t *mo;
     actor->special2.i = 5;        // Raise speed
@@ -1102,7 +1099,7 @@ void A_ThrustInitDn(mobj_t * actor)
 }
 
 
-void A_ThrustRaise(mobj_t * actor)
+void A_ThrustRaise(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (A_RaiseMobj(actor))
     {                           // Reached it's target height
@@ -1126,7 +1123,7 @@ void A_ThrustRaise(mobj_t * actor)
     actor->special2.i++;          // Increase raise speed
 }
 
-void A_ThrustLower(mobj_t * actor)
+void A_ThrustLower(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (A_SinkMobj(actor))
     {
@@ -1138,12 +1135,12 @@ void A_ThrustLower(mobj_t * actor)
     }
 }
 
-void A_ThrustBlock(mobj_t * actor)
+void A_ThrustBlock(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     actor->flags |= MF_SOLID;
 }
 
-void A_ThrustImpale(mobj_t * actor)
+void A_ThrustImpale(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     // Impale all shootables in radius
     PIT_ThrustSpike(actor);
@@ -1155,7 +1152,7 @@ void A_ThrustImpale(mobj_t * actor)
 //
 //===========================================================================
 
-void A_SoAExplode(mobj_t * actor)
+void A_SoAExplode(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     mobj_t *mo;
     int i;
@@ -1177,11 +1174,9 @@ void A_SoAExplode(mobj_t * actor)
     }
     if (actor->args[0])
     {                           // Spawn an item
-#if 0 // Checks are not present in version 1.1
-        if (!nomonsters
+        if ((gameversion != exe_hexen_1_1r2) || !nomonsters
             || !(mobjinfo[TranslateThingType[actor->args[0]]].
                  flags & MF_COUNTKILL))
-#endif
         {                       // Only spawn monsters if not -nomonsters
             P_SpawnMobj(actor->x, actor->y, actor->z,
                         TranslateThingType[actor->args[0]]);
@@ -1197,7 +1192,7 @@ void A_SoAExplode(mobj_t * actor)
 //
 //===========================================================================
 
-void A_BellReset1(mobj_t * actor)
+void A_BellReset1(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     actor->flags |= MF_NOGRAVITY;
     actor->height <<= 2;
@@ -1209,7 +1204,7 @@ void A_BellReset1(mobj_t * actor)
 //
 //===========================================================================
 
-void A_BellReset2(mobj_t * actor)
+void A_BellReset2(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     actor->flags |= MF_SHOOTABLE;
     actor->flags &= ~MF_CORPSE;
@@ -1223,7 +1218,7 @@ void A_BellReset2(mobj_t * actor)
 //
 //===========================================================================
 
-void A_FlameCheck(mobj_t * actor)
+void A_FlameCheck(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (!actor->args[0]--)      // Called every 8 tics
     {
@@ -1247,12 +1242,12 @@ void A_FlameCheck(mobj_t * actor)
 //      args[4]         turn amount per move (in degrees)
 //===========================================================================
 
-void A_BatSpawnInit(mobj_t * actor)
+void A_BatSpawnInit(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     actor->special1.i = 0;        // Frequency count
 }
 
-void A_BatSpawn(mobj_t * actor)
+void A_BatSpawn(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     mobj_t *mo;
     int delta;
@@ -1278,7 +1273,7 @@ void A_BatSpawn(mobj_t * actor)
 }
 
 
-void A_BatMove(mobj_t * actor)
+void A_BatMove(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     angle_t newangle;
     fixed_t speed;
@@ -1318,7 +1313,7 @@ void A_BatMove(mobj_t * actor)
 //
 //===========================================================================
 
-void A_TreeDeath(mobj_t * actor)
+void A_TreeDeath(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (!(actor->flags2 & MF2_FIREDAMAGE))
     {
@@ -1340,7 +1335,7 @@ void A_TreeDeath(mobj_t * actor)
 //
 //===========================================================================
 
-void A_NoGravity(mobj_t * actor)
+void A_NoGravity(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     actor->flags |= MF_NOGRAVITY;
 }
