@@ -33,15 +33,13 @@
 int cheating = 0;
 static int grid = 0;
 
-static int leveljuststarted = 1;        // kluge until AM_LevelInit() is called
-
 boolean automapactive = false;
 static int finit_width;// = SCREENWIDTH;
 static int finit_height;// = SCREENHEIGHT - SBARHEIGHT - (3 << crispy->hires);
 static int f_x, f_y;            // location of window on screen
 static int f_w, f_h;            // size of window on screen
 static int lightlev;            // used for funky strobing effect
-static byte *fb;                // pseudo-frame buffer
+static pixel_t *fb;             // pseudo-frame buffer
 static int amclock;
 
 static mpoint_t m_paninc;       // how far the window pans each tic (map coords)
@@ -102,9 +100,9 @@ static byte antialias_normal[NUMALIAS][8] = {
 
 // [crispy] gradient table for map overlay mode
 static byte antialias_overlay[NUMALIAS][8] = {
-    {86, 85, 84, 83, 82, 81, 100, 99},
-    {96, 93, 90, 87, 85, 83, 81, 99},
-    {107, 105, 104, 103, 102, 101, 100, 99}
+    {86, 85, 85, 84, 83, 83, 82, 81},
+    {93, 93, 92, 91, 90, 89, 88, 87},
+    {107, 107, 106, 105, 104, 103, 102, 101}
 };
 
 static byte (*antialias)[NUMALIAS][8]; // [crispy]
@@ -130,10 +128,6 @@ static void AM_rotatePoint(mpoint_t *pt);
 static mpoint_t mapcenter;
 static angle_t mapangle;
 
-// [AM] Fractional part of the current tic, in the half-open
-//      range of [0.0, 1.0).  Used for interpolation.
-extern fixed_t          fractionaltic;
-
 //byte screens[][SCREENWIDTH*SCREENHEIGHT];
 //void V_MarkRect (int x, int y, int width, int height);
 
@@ -460,8 +454,6 @@ void AM_LevelInit(boolean reinit)
     // [crispy] Used for reinit
     static int f_h_old;
 
-    leveljuststarted = 0;
-
     finit_width = SCREENWIDTH;
     finit_height = SCREENHEIGHT - ((ORIGSBARHEIGHT + 3) << crispy->hires);
 
@@ -997,6 +989,7 @@ void AM_clearFB(int color)
 
     for (i = 0; i < SCREENHEIGHT - SBARHEIGHT; i++)
     {
+#ifndef CRISPY_TRUECOLOR
         memcpy(I_VideoBuffer + i * finit_width,
                maplump + j + (MAPBGROUNDWIDTH << crispy->hires) - x3, x3);
 
@@ -1005,6 +998,30 @@ void AM_clearFB(int color)
 
         memcpy(I_VideoBuffer + i * finit_width + x2 + x3,
                maplump + j, x1);
+#else
+        int z;
+        pixel_t *dest = I_VideoBuffer + i * finit_width;
+        byte *src = maplump + j + (MAPBGROUNDWIDTH << crispy->hires) - x3;
+
+        for (z = 0; z < x3; z++)
+        {
+            dest[z] = pal_color[src[z]];
+        }
+
+        dest += x3;
+        src += x3 - x2;
+        for (z = 0; z < x2; z++)
+        {
+            dest[z] = pal_color[src[z]];
+        }
+
+        dest += x2;
+        src = maplump + j;
+        for (z = 0; z < x1; z++)
+        {
+            dest[z] = pal_color[src[z]];
+        }
+#endif
 
         j += MAPBGROUNDWIDTH << crispy->hires;
         if (j >= MAPBGROUNDHEIGHT * MAPBGROUNDWIDTH)
@@ -1157,7 +1174,11 @@ void AM_drawFline(fline_t * fl, int color)
                     return;
                 }
 
+#ifndef CRISPY_TRUECOLOR
 #define DOT(xx,yy,cc) fb[(yy)*f_w+(xx)]=(cc)    //the MACRO!
+#else
+#define DOT(xx,yy,cc) fb[(yy)*f_w+(xx)]=(pal_color[cc])
+#endif
 
                 dx = fl->b.x - fl->a.x;
                 ax = 2 * (dx < 0 ? -dx : dx);
@@ -1257,7 +1278,11 @@ void PUTDOT(short xx, short yy, byte * cc, byte * cm)
         oldyy = yy;
         oldyyshifted = yy * f_w;
     }
+#ifndef CRISPY_TRUECOLOR
     fb[oldyyshifted + xx] = *(cc);
+#else
+    fb[oldyyshifted + xx] = pal_color[*(cc)];
+#endif
 //      fb[(yy)*f_w+(xx)]=*(cc);
 }
 
@@ -1714,8 +1739,8 @@ void AM_drawThings(int colors, int colorrange)
             // [crispy] interpolate thing triangles movement
             if (leveltime > oldleveltime)
             {
-            pt.x = (t->oldx + FixedMul(t->x - t->oldx, fractionaltic)) >> FRACTOMAPBITS;
-            pt.y = (t->oldy + FixedMul(t->y - t->oldy, fractionaltic)) >> FRACTOMAPBITS;
+            pt.x = LerpFixed(t->oldx, t->x) >> FRACTOMAPBITS;
+            pt.y = LerpFixed(t->oldy, t->y) >> FRACTOMAPBITS;
             }
             else
             {
