@@ -26,6 +26,7 @@
 #include "v_video.h"
 #include "i_swap.h"
 #include "am_map.h"
+#include "v_trans.h" // [crispy] blending functions
 
 
 // MACROS ------------------------------------------------------------------
@@ -61,9 +62,14 @@ static int FinaleLumpNum;
 static int FontABaseLump;
 static char *FinaleText;
 
+#ifndef CRISPY_TRUECOLOR
 static fixed_t *Palette;
 static fixed_t *PaletteDelta;
 static byte *RealPalette;
+#else
+static int BlendTic; // [crispy] tics representing opacity value for blending function
+#define BLENDSTEP  4 // [crispy] step of increasing/decreasing opacity value
+#endif
 
 // CODE --------------------------------------------------------------------
 
@@ -153,6 +159,26 @@ void F_Ticker(void)
     {
         FadePic();
     }
+#ifdef CRISPY_TRUECOLOR
+    // [crispy] Fade in (from-black-to-normal) on 0 and 4 stages,
+    // and fade out (from normal-to-black) on 3 stage.
+    if (FinaleStage == 0 || FinaleStage == 4)
+    {
+        BlendTic += BLENDSTEP;
+        if (BlendTic > 255)
+        {
+            BlendTic = 255;
+        }
+    }
+    if (FinaleStage == 3)
+    {
+        BlendTic -= BLENDSTEP;
+        if (BlendTic < 0)
+        {
+            BlendTic = 0;
+        }
+    }
+#endif
 }
 
 //===========================================================================
@@ -240,6 +266,7 @@ static void TextWrite(void)
 
 static void InitializeFade(boolean fadeIn)
 {
+#ifndef CRISPY_TRUECOLOR
     unsigned i;
 
     Palette = Z_Malloc(768 * sizeof(fixed_t), PU_STATIC, 0);
@@ -268,6 +295,10 @@ static void InitializeFade(boolean fadeIn)
         }
     }
     I_SetPalette(RealPalette);
+#else
+    // [crispy] reset blending tic to initial value
+    BlendTic = fadeIn ? 0 : 255;
+#endif
 }
 
 //===========================================================================
@@ -278,9 +309,11 @@ static void InitializeFade(boolean fadeIn)
 
 static void DeInitializeFade(void)
 {
+#ifndef CRISPY_TRUECOLOR
     Z_Free(Palette);
     Z_Free(PaletteDelta);
     Z_Free(RealPalette);
+#endif
 }
 
 //===========================================================================
@@ -291,6 +324,7 @@ static void DeInitializeFade(void)
 
 static void FadePic(void)
 {
+#ifndef CRISPY_TRUECOLOR
     unsigned i;
 
     for (i = 0; i < 768; i++)
@@ -299,6 +333,7 @@ static void FadePic(void)
         RealPalette[i] = Palette[i] >> FRACBITS;
     }
     I_SetPalette(RealPalette);
+#endif
 }
 
 //===========================================================================
@@ -323,6 +358,13 @@ static void DrawPic(void)
                                               1, PU_CACHE));
         }
     }
+#ifdef CRISPY_TRUECOLOR
+    // [crispy] apply true color blending on top of patch drawing functions
+    for (int y = 0; y < SCREENWIDTH * SCREENHEIGHT; y++)
+    {
+        I_VideoBuffer[y] = I_BlendDark(I_VideoBuffer[y], BlendTic);
+    }
+#endif
 }
 
 //===========================================================================
