@@ -290,7 +290,6 @@ static void TryPickupWeapon(player_t * player, pclass_t weaponClass,
     if (player == &players[consoleplayer])
     {
         S_StartSound(NULL, SFX_PICKUP_WEAPON);
-        SB_PaletteFlash(false);
     }
 }
 
@@ -492,10 +491,6 @@ static void TryPickupWeaponPiece(player_t * player, pclass_t matchClass,
         }
     }
     player->bonuscount += BONUSADD;
-    if (player == &players[consoleplayer])
-    {
-        SB_PaletteFlash(false);
-    }
 
     // Check if fourth weapon assembled
     if (checkAssembled)
@@ -845,9 +840,9 @@ boolean P_GiveArtifact(player_t * player, artitype_t arti, mobj_t * mo)
     {
         inv_ptr++;
         curpos++;
-        if (curpos > 6)
+        if (curpos > CURPOS_MAX)
         {
-            curpos = 6;
+            curpos = CURPOS_MAX;
         }
     }
     player->artifactCount++;
@@ -1027,7 +1022,6 @@ void P_TouchSpecialThing(mobj_t * special, mobj_t * toucher)
             if (player == &players[consoleplayer])
             {
                 S_StartSound(NULL, sound);
-                SB_PaletteFlash(false);
             }
             return;
 
@@ -1241,7 +1235,6 @@ void P_TouchSpecialThing(mobj_t * special, mobj_t * toucher)
     if (player == &players[consoleplayer])
     {
         S_StartSound(NULL, sound);
-        SB_PaletteFlash(false);
     }
 }
 
@@ -1985,7 +1978,6 @@ void P_DamageMobj
         if (player == &players[consoleplayer])
         {
             I_Tactile(40, 10, 40 + temp * 2);
-            SB_PaletteFlash(false);
         }
     }
 
