@@ -76,7 +76,7 @@ line_t *ceilingline;
 
 // keep track of special lines as they are hit, but don't process them
 // until the move is proven valid
-#define MAXSPECIALCROSS 8
+
 line_t *spechit[MAXSPECIALCROSS];
 int numspechit;
 
@@ -189,6 +189,10 @@ boolean P_TeleportMove(mobj_t * thing, fixed_t x, fixed_t y)
     thing->x = x;
     thing->y = y;
 
+    // [AM] Don't interpolate mobjs that pass
+    //      through teleporters
+    thing->interp = false;
+
     P_SetThingPosition(thing);
 
     return true;
@@ -1570,7 +1574,6 @@ fixed_t attackrange;
 
 fixed_t aimslope;
 
-extern fixed_t topslope, bottomslope;   // slopes to top and bottom of target
 
 /*
 ===============================================================================
@@ -1681,8 +1684,6 @@ boolean PTR_ShootTraverse(intercept_t * in)
     fixed_t dist;
     fixed_t thingtopslope, thingbottomslope;
 
-    extern mobj_t LavaInflictor;
-
     if (in->isaline)
     {
         li = in->d.line;
