@@ -1187,9 +1187,13 @@ void A_MStaffAttack(mobj_t *actor, player_t *player, pspdef_t *psp)
     {
         player->damagecount = 0;
         player->bonuscount = 0;
+#ifndef CRISPY_TRUECOLOR
         I_SetPalette((byte *) W_CacheLumpNum(W_GetNumForName("playpal"),
                                              PU_CACHE) +
                      STARTSCOURGEPAL * 768);
+#else
+        I_SetPalette(STARTSCOURGEPAL);
+#endif
     }
 }
 
@@ -1210,8 +1214,12 @@ void A_MStaffPalette(mobj_t *actor, player_t *player, pspdef_t *psp)
         {                       // reset back to original playpal
             pal = 0;
         }
+#ifndef CRISPY_TRUECOLOR
         I_SetPalette((byte *) W_CacheLumpNum(W_GetNumForName("playpal"),
                                              PU_CACHE) + pal * 768);
+#else
+        I_SetPalette(pal);
+#endif
     }
 }
 
@@ -1929,8 +1937,12 @@ void A_CHolyAttack(mobj_t *actor, player_t *player, pspdef_t *psp)
     {
         player->damagecount = 0;
         player->bonuscount = 0;
+#ifndef CRISPY_TRUECOLOR
         I_SetPalette((byte *) W_CacheLumpNum(W_GetNumForName("playpal"),
                                              PU_CACHE) + STARTHOLYPAL * 768);
+#else
+        I_SetPalette(STARTHOLYPAL);
+#endif
     }
     S_StartSound(player->mo, SFX_CHOLY_FIRE);
 }
@@ -1952,8 +1964,12 @@ void A_CHolyPalette(mobj_t *actor, player_t *player, pspdef_t *psp)
         {                       // reset back to original playpal
             pal = 0;
         }
+#ifndef CRISPY_TRUECOLOR
         I_SetPalette((byte *) W_CacheLumpNum(W_GetNumForName("playpal"),
                                              PU_CACHE) + pal * 768);
+#else
+        I_SetPalette(pal);
+#endif
     }
 }
 
@@ -2480,6 +2496,11 @@ void P_MovePsprites(player_t * player)
     psp->sy2 = psp->sy;
     if (psp->state && (crispy->bobfactor || crispy->centerweapon || crispy->uncapped))
     {
+        // [crispy] don't align swiping weapons
+        const boolean swiping_weapon = player->class == PCLASS_FIGHTER ||
+                                       (player->class == PCLASS_CLERIC &&
+                                        player->readyweapon == WP_FIRST);
+
         // [crispy] don't center vertically during lowering and raising states
         if (psp->state->action == A_Lower || psp->state->action == A_Raise)
         {
@@ -2487,7 +2508,7 @@ void P_MovePsprites(player_t * player)
         else
         // [crispy] not attacking means idle
         if (!player->attackdown ||
-            crispy->centerweapon == CENTERWEAPON_BOB)
+            (crispy->centerweapon == CENTERWEAPON_BOB && !swiping_weapon))
         {
             angle_t angle = (128 * leveltime) & FINEMASK;
             psp->sx2 = FRACUNIT + FixedMul(player->bob2, finecosine[angle]);
@@ -2496,7 +2517,7 @@ void P_MovePsprites(player_t * player)
         }
         else
         // [crispy] center the weapon sprite horizontally and push up vertically
-        if (crispy->centerweapon == CENTERWEAPON_CENTER)
+        if (crispy->centerweapon == CENTERWEAPON_CENTER && !swiping_weapon)
         {
             psp->sx2 = FRACUNIT;
             psp->sy2 = WEAPONTOP;
