@@ -18,6 +18,7 @@
 #include "SDL.h"
 #include <string.h>
 
+#include "crispy.h"
 #include "doomtype.h"
 
 #include "deh_str.h"
@@ -33,7 +34,7 @@
 static boolean pcs_initialized = false;
 
 static SDL_mutex *sound_lock;
-static boolean use_sfx_prefix;
+static GameMission_t gamemission;
 
 static uint8_t *current_sound_lump = NULL;
 static uint8_t *current_sound_pos = NULL;
@@ -158,8 +159,15 @@ static boolean IsDisabledSound(sfxinfo_t *sfxinfo)
         "dmpain",
         "popain",
         "sawidl",
+        "rifle",
     };
 
+    // [crispy] Restore missing assault rifle PC speaker sound.
+    if (gamemission == strife && crispy->soundfix)
+    {
+        return false;
+    }
+
     for (i=0; i<arrlen(disabled_sounds); ++i)
     {
         if (!strcmp(sfxinfo->name, disabled_sounds[i]))
@@ -244,9 +252,15 @@ static int I_PCS_GetSfxLumpNum(sfxinfo_t* sfx)
 {
     char namebuf[9];
 
-    if (use_sfx_prefix)
+    if (gamemission == doom || gamemission == strife)
     {
         M_snprintf(namebuf, sizeof(namebuf), "dp%s", DEH_String(sfx->name));
+
+        if (gamemission == strife && W_CheckNumForName(namebuf) == -1)
+        {
+            // Missing sounds replaced with DPRIFLE.
+            M_snprintf(namebuf, sizeof(namebuf), "dp%s", DEH_String("rifle"));
+        }
     }
     else
     {
@@ -273,9 +287,9 @@ static boolean I_PCS_SoundIsPlaying(int handle)
     return current_sound_lump != NULL && current_sound_remaining > 0;
 }
 
-static boolean I_PCS_InitSound(boolean _use_sfx_prefix)
+static boolean I_PCS_InitSound(GameMission_t mission)
 {
-    use_sfx_prefix = _use_sfx_prefix;
+    gamemission = mission;
 
     // Use the sample rate from the configuration file
 
