@@ -96,10 +96,14 @@ static SDL_Texture *curpane = NULL;
 static SDL_Texture *redpane = NULL;
 static SDL_Texture *yelpane = NULL;
 static SDL_Texture *grnpane = NULL;
+// Hexen exclusive color panes
+static SDL_Texture *grnspane = NULL;
+static SDL_Texture *bluepane = NULL;
+static SDL_Texture *graypane = NULL;
+static SDL_Texture *orngpane = NULL;
 static int pane_alpha;
 static unsigned int rmask, gmask, bmask, amask; // [crispy] moved up here
-static const uint8_t blend_alpha = 0xa8;
-extern pixel_t* colormaps; // [crispy] evil hack to get FPS dots working as in Vanilla
+extern pixel_t* pal_color; // [crispy] evil hack to get FPS dots working as in Vanilla
 #else
 static SDL_Color palette[256];
 #endif
@@ -533,6 +537,18 @@ void I_StartTic (void)
     }
 }
 
+void I_StartDisplay(void) // [crispy]
+{
+    // [AM] Figure out how far into the current tic we're in as a fixed_t.
+    fractionaltic = I_GetFracRealTime();
+
+    SDL_PumpEvents();
+
+    if (usemouse && !nomouse && window_focused)
+    {
+        I_ReadMouseUncapped();
+    }
+}
 
 //
 // I_UpdateNoBlit
@@ -801,13 +817,13 @@ void I_FinishUpdate (void)
 #ifndef CRISPY_TRUECOLOR
 	    I_VideoBuffer[ (SCREENHEIGHT-1)*SCREENWIDTH + i] = 0xff;
 #else
-	    I_VideoBuffer[ (SCREENHEIGHT-1)*SCREENWIDTH + i] = colormaps[0xff];
+	    I_VideoBuffer[ (SCREENHEIGHT-1)*SCREENWIDTH + i] = pal_color[0xff];
 #endif
 	for ( ; i<20*4 ; i+=4)
 #ifndef CRISPY_TRUECOLOR
 	    I_VideoBuffer[ (SCREENHEIGHT-1)*SCREENWIDTH + i] = 0x0;
 #else
-	    I_VideoBuffer[ (SCREENHEIGHT-1)*SCREENWIDTH + i] = colormaps[0x0];
+	    I_VideoBuffer[ (SCREENHEIGHT-1)*SCREENWIDTH + i] = pal_color[0x0];
 #endif
     }
 
@@ -894,7 +910,7 @@ void I_FinishUpdate (void)
 
     SDL_RenderPresent(renderer);
 
-    if (crispy->uncapped)
+    if (crispy->uncapped && !singletics)
     {
         // Limit framerate
         if (crispy->fpslimit >= TICRATE)
@@ -922,9 +938,6 @@ void I_FinishUpdate (void)
                 }
             }
         }
-
-        // [AM] Figure out how far into the current tic we're in as a fixed_t.
-        fractionaltic = I_GetFracRealTime();
     }
 
     // Restore background and undo the disk indicator, if it was drawn.
@@ -945,29 +958,38 @@ void I_ReadScreen (pixel_t* scr)
 // I_SetPalette
 //
 // [crispy] intermediate gamma levels
-byte **gamma2table = NULL;
+byte gamma2table[18][256];
+
+static const float gammalevels[9] =
+{
+    // Darker
+    0.50f, 0.55f, 0.60f, 0.65f, 0.70f, 0.75f, 0.80f, 0.85f, 0.90f,
+};
+
 void I_SetGammaTable (void)
 {
-	int i;
+	int i, j, k;
 
-	gamma2table = malloc(9 * sizeof(*gamma2table));
+	for (i = 0; i < 9; ++i)
+	{
+		for (j = 0; j < 256; ++j)
+		{
+			gamma2table[i][j] = (byte)(pow(j / 255.0, 1.0 / gammalevels[i]) * 255.0 + 0.5);
+		}
+	}
 
 	// [crispy] 5 original gamma levels
-	for (i = 0; i < 5; i++)
+	for (i = 9, k = 0; i < 18 && k < 5; i += 2, k++)
 	{
-		gamma2table[2*i] = (byte *)gammatable[i];
+		memcpy(gamma2table[i], gammatable[k], 256);
 	}
 
 	// [crispy] 4 intermediate gamma levels
-	for (i = 0; i < 4; i++)
+	for (i = 10, k = 0; i < 18 && k < 4; i += 2, k++)
 	{
-		int j;
-
-		gamma2table[2*i+1] = malloc(256 * sizeof(**gamma2table));
-
 		for (j = 0; j < 256; j++)
 		{
-			gamma2table[2*i+1][j] = (gamma2table[2*i][j] + gamma2table[2*i+2][j]) / 2;
+			gamma2table[i][j] = (gammatable[k][j] + gammatable[k + 1][j]) / 2;
 		}
 	}
 }
@@ -977,21 +999,15 @@ void I_SetPalette (byte *doompalette)
 {
     int i;
 
-    // [crispy] intermediate gamma levels
-    if (!gamma2table)
-    {
-        I_SetGammaTable();
-    }
-
     for (i=0; i<256; ++i)
     {
         // Zero out the bottom two bits of each channel - the PC VGA
         // controller only supports 6 bits of accuracy.
 
         // [crispy] intermediate gamma levels
-        palette[i].r = gamma2table[usegamma][*doompalette++] & ~3;
-        palette[i].g = gamma2table[usegamma][*doompalette++] & ~3;
-        palette[i].b = gamma2table[usegamma][*doompalette++] & ~3;
+        palette[i].r = gamma2table[crispy->gamma][*doompalette++] & ~3;
+        palette[i].g = gamma2table[crispy->gamma][*doompalette++] & ~3;
+        palette[i].b = gamma2table[crispy->gamma][*doompalette++] & ~3;
     }
 
     palette_to_set = true;
@@ -1056,6 +1072,64 @@ void I_SetPalette (int palette)
 	    curpane = grnpane;
 	    pane_alpha = 0xff * 125 / 1000;
 	    break;
+	// Hexen exclusive color panes and palette indexes
+	// https://doomwiki.org/wiki/PLAYPAL#Hexen
+	case 14:  // STARTPOISONPALS + 1 (13 is shared with other games)
+	    curpane = grnspane;
+	    pane_alpha = 0x33; // 51 (20%)
+	    break;
+	case 15:
+	    curpane = grnspane;
+	    pane_alpha = 0x4c; // 76 (30%)
+	    break;
+	case 16:
+	    curpane = grnspane;
+	    pane_alpha = 0x66; // 102 (40%)
+	    break;
+	case 17:
+	    curpane = grnspane;
+	    pane_alpha = 0x7f; // 127 (50%)
+	    break;
+	case 18:
+	    curpane = grnspane;
+	    pane_alpha = 0x99; // 153 (60%)
+	    break;
+	case 19:
+	    curpane = grnspane;
+	    pane_alpha = 0xb2; // 178 (70%)
+	    break;
+	case 20:
+	    curpane = grnspane;
+	    pane_alpha = 0xcc; // 204 (80%)
+	    break;
+	case 21:  // STARTICEPAL
+	    curpane = bluepane;
+	    pane_alpha = 0x80; // 128 (50%)
+	    break;
+	case 22:  // STARTHOLYPAL
+	    curpane = graypane;
+	    pane_alpha = 0x7f; // 127 (50%)
+	    break;
+	case 23:
+	    curpane = graypane;
+	    pane_alpha = 0x6a; // 106
+	    break;
+	case 24:
+	    curpane = graypane;
+	    pane_alpha = 0x34; // 52
+	    break;
+	case 25:  // STARTSCOURGEPAL
+	    curpane = orngpane;
+	    pane_alpha = 0x7f; // 127 (50%)
+	    break;
+	case 26:
+	    curpane = orngpane;
+	    pane_alpha = 0x60; // 96
+	    break;
+	case 27:
+	    curpane = orngpane;
+	    pane_alpha = 0x48; // 72
+	    break;
 	default:
 	    I_Error("Unknown palette: %d!\n", palette);
 	    break;
@@ -1112,10 +1186,20 @@ void I_InitWindowIcon(void)
 
 static void SetScaleFactor(int factor)
 {
+    int height;
+
     // Pick 320x200 or 320x240, depending on aspect ratio correct
+    if (aspect_ratio_correct)
+    {
+        height = SCREENHEIGHT_4_3;
+    }
+    else
+    {
+        height = SCREENHEIGHT;
+    }
 
     window_width = factor * SCREENWIDTH;
-    window_height = factor * actualheight;
+    window_height = factor * height;
     fullscreen = false;
 }
 
@@ -1452,7 +1536,7 @@ static void SetVideoMode(void)
     }
 
     // Turn on vsync if we aren't in a -timedemo
-    if (!singletics && mode.refresh_rate > 0)
+    if ((!singletics && mode.refresh_rate > 0) || crispy->demowarp)
     {
         if (crispy->vsync) // [crispy] uncapped vsync
         {
@@ -1568,6 +1652,22 @@ static void SetVideoMode(void)
         SDL_FillRect(argbbuffer, NULL, I_MapRGB(0x0, 0xff, 0x0));
         grnpane = SDL_CreateTextureFromSurface(renderer, argbbuffer);
         SDL_SetTextureBlendMode(grnpane, SDL_BLENDMODE_BLEND);
+
+        SDL_FillRect(argbbuffer, NULL, I_MapRGB(0x2c, 0x5c, 0x24)); // 44, 92, 36
+        grnspane = SDL_CreateTextureFromSurface(renderer, argbbuffer);
+        SDL_SetTextureBlendMode(grnspane, SDL_BLENDMODE_BLEND);
+
+        SDL_FillRect(argbbuffer, NULL, I_MapRGB(0x0, 0x0, 0xe0)); // 0, 0, 224
+        bluepane = SDL_CreateTextureFromSurface(renderer, argbbuffer);
+        SDL_SetTextureBlendMode(bluepane, SDL_BLENDMODE_BLEND);
+
+        SDL_FillRect(argbbuffer, NULL, I_MapRGB(0x82, 0x82, 0x82)); // 130, 130, 130
+        graypane = SDL_CreateTextureFromSurface(renderer, argbbuffer);
+        SDL_SetTextureBlendMode(graypane, SDL_BLENDMODE_BLEND);
+
+        SDL_FillRect(argbbuffer, NULL, I_MapRGB(0x96, 0x6e, 0x0)); // 150, 110, 0
+        orngpane = SDL_CreateTextureFromSurface(renderer, argbbuffer);
+        SDL_SetTextureBlendMode(orngpane, SDL_BLENDMODE_BLEND);
 #endif
         SDL_FillRect(argbbuffer, NULL, 0);
     }
@@ -1664,6 +1764,16 @@ void I_GetScreenDimensions (void)
 	WIDESCREENDELTA = ((SCREENWIDTH - NONWIDEWIDTH) >> crispy->hires) / 2;
 }
 
+// [crispy] calls native SDL vsync toggle
+void I_ToggleVsync (void)
+{
+#if SDL_VERSION_ATLEAST(2, 0, 18)
+    SDL_RenderSetVSync(renderer, crispy->vsync);
+#else
+    I_ReInitGraphics(REINIT_RENDERER | REINIT_TEXTURES | REINIT_ASPECTRATIO);
+#endif
+}
+
 void I_InitGraphics(void)
 {
     SDL_Event dummy;
@@ -2032,16 +2142,47 @@ const pixel_t I_BlendDark (const pixel_t bg, const int d)
 	return amask | sag | srb;
 }
 
-const pixel_t I_BlendOver (const pixel_t bg, const pixel_t fg)
+// [crispy] Main overlay blending function
+const pixel_t I_BlendOver (const pixel_t bg, const pixel_t fg, const int amount)
 {
-	const uint32_t r = ((blend_alpha * (fg & rmask) + (0xff - blend_alpha) * (bg & rmask)) >> 8) & rmask;
-	const uint32_t g = ((blend_alpha * (fg & gmask) + (0xff - blend_alpha) * (bg & gmask)) >> 8) & gmask;
-	const uint32_t b = ((blend_alpha * (fg & bmask) + (0xff - blend_alpha) * (bg & bmask)) >> 8) & bmask;
+	const uint32_t r = ((amount * (fg & rmask) + (0xff - amount) * (bg & rmask)) >> 8) & rmask;
+	const uint32_t g = ((amount * (fg & gmask) + (0xff - amount) * (bg & gmask)) >> 8) & gmask;
+	const uint32_t b = ((amount * (fg & bmask) + (0xff - amount) * (bg & bmask)) >> 8) & bmask;
 
 	return amask | r | g | b;
 }
 
-const pixel_t (*blendfunc) (const pixel_t fg, const pixel_t bg) = I_BlendOver;
+// [crispy] TRANMAP blending emulation, used for Doom
+const pixel_t I_BlendOverTranmap (const pixel_t bg, const pixel_t fg)
+{
+	return I_BlendOver(bg, fg, 0xA8); // 168 (66% opacity)
+}
+
+// [crispy] TINTTAB blending emulation, used for Heretic and Hexen
+const pixel_t I_BlendOverTinttab (const pixel_t bg, const pixel_t fg)
+{
+	return I_BlendOver(bg, fg, 0x60); // 96 (38% opacity)
+}
+
+// [crispy] More opaque ("Alt") TINTTAB blending emulation, used for Hexen's MF_ALTSHADOW drawing
+const pixel_t I_BlendOverAltTinttab (const pixel_t bg, const pixel_t fg)
+{
+	return I_BlendOver(bg, fg, 0x8E); // 142 (56% opacity)
+}
+
+// [crispy] More opaque XLATAB blending emulation, used for Strife
+const pixel_t I_BlendOverXlatab (const pixel_t bg, const pixel_t fg)
+{
+	return I_BlendOver(bg, fg, 0xC0); // 192 (75% opacity)
+}
+
+// [crispy] Less opaque ("Alt") XLATAB blending emulation, used for Strife
+const pixel_t I_BlendOverAltXlatab (const pixel_t bg, const pixel_t fg)
+{
+	return I_BlendOver(bg, fg, 0x40); // 64 (25% opacity)
+}
+
+const pixel_t (*blendfunc) (const pixel_t fg, const pixel_t bg) = I_BlendOverTranmap;
 
 const pixel_t I_MapRGB (const uint8_t r, const uint8_t g, const uint8_t b)
 {
