@@ -1095,6 +1095,13 @@ static default_t extra_defaults_list[] =
 
     CONFIG_VARIABLE_FLOAT(fsynth_reverb_width),
 
+    //!
+    // Fine tune the FluidSynth output level. Default is 1.0,
+    // range is 0.0 - 10.0.
+    //
+
+    CONFIG_VARIABLE_FLOAT(fsynth_gain),
+
     //!
     // Full path to a soundfont file to use with FluidSynth MIDI playback.
     //
@@ -1132,9 +1139,16 @@ static default_t extra_defaults_list[] =
 
     CONFIG_VARIABLE_STRING(winmm_midi_device),
 
+    //!
+    // Compatibility level for native Windows MIDI, default 1. Valid values are
+    // 0 (Vanilla), 1 (Standard), 2 (Full).
+    //
+
+    CONFIG_VARIABLE_INT(winmm_complevel),
+
     //!
     // Reset device type for native Windows MIDI, default 1. Valid values are
-    // 0 (None), 1 (GS Mode), 2 (GM Mode), 3 (GM2 Mode), 4 (XG Mode).
+    // 0 (None), 1 (GM Mode), 2 (GS Mode), 3 (XG Mode).
     //
 
     CONFIG_VARIABLE_INT(winmm_reset_type),
@@ -1144,18 +1158,6 @@ static default_t extra_defaults_list[] =
     //
 
     CONFIG_VARIABLE_INT(winmm_reset_delay),
-
-    //!
-    // Reverb level for native Windows MIDI, default -1, range 0-127.
-    //
-
-    CONFIG_VARIABLE_INT(winmm_reverb_level),
-
-    //!
-    // Chorus level for native Windows MIDI, default -1, range 0-127.
-    //
-
-    CONFIG_VARIABLE_INT(winmm_chorus_level),
 #endif
 
     //!
@@ -1376,6 +1378,12 @@ static default_t extra_defaults_list[] =
 
     CONFIG_VARIABLE_INT(joystick_index),
 
+    //!
+    // If non-zero, use analog movement when playing with a gamepad.
+    //
+
+    CONFIG_VARIABLE_INT(use_analog),
+
     //!
     // Joystick axis to use to for horizontal (X) movement.
     //
@@ -1388,6 +1396,12 @@ static default_t extra_defaults_list[] =
 
     CONFIG_VARIABLE_INT(joystick_x_invert),
 
+    //!
+    // Joystick turn analog sensitivity, specified as a value between 0 and 20.
+    //
+
+    CONFIG_VARIABLE_INT(joystick_turn_sensitivity),
+
     //!
     // Joystick axis to use to for vertical (Y) movement.
     //
@@ -1413,6 +1427,12 @@ static default_t extra_defaults_list[] =
 
     CONFIG_VARIABLE_INT(joystick_strafe_invert),
 
+    //!
+    // Joystick move and strafe analog sensitivity, specified as a value
+    // between 0 and 20.
+    //
+
+    CONFIG_VARIABLE_INT(joystick_move_sensitivity),
     //!
     // Joystick axis to use to for looking up and down.
     //
@@ -1426,6 +1446,12 @@ static default_t extra_defaults_list[] =
 
     CONFIG_VARIABLE_INT(joystick_look_invert),
 
+    //!
+    // Joystick look analog sensitivity, specified as a value between 0 and 20.
+    //
+
+    CONFIG_VARIABLE_INT(joystick_look_sensitivity),
+
     //!
     // The physical joystick button that corresponds to joystick
     // virtual button #0.
@@ -1503,6 +1529,89 @@ static default_t extra_defaults_list[] =
 
     CONFIG_VARIABLE_INT(joystick_physical_button10),
 
+    //!
+    // The physical joystick button that corresponds to joystick
+    // virtual button #11.
+    //
+
+    CONFIG_VARIABLE_INT(joystick_physical_button11),
+
+    //!
+    // The physical joystick button that corresponds to joystick
+    // virtual button #12.
+    //
+
+    CONFIG_VARIABLE_INT(joystick_physical_button12),
+
+    //!
+    // The physical joystick button that corresponds to joystick
+    // virtual button #13.
+    //
+
+    CONFIG_VARIABLE_INT(joystick_physical_button13),
+
+    //!
+    // The physical joystick button that corresponds to joystick
+    // virtual button #14.
+    //
+
+    CONFIG_VARIABLE_INT(joystick_physical_button14),
+
+    //!
+    // The physical joystick button that corresponds to joystick
+    // virtual button #15.
+    //
+
+    CONFIG_VARIABLE_INT(joystick_physical_button15),
+
+    //!
+    // The physical joystick button that corresponds to joystick
+    // virtual button #16.
+    //
+
+    CONFIG_VARIABLE_INT(joystick_physical_button16),
+
+    //!
+    // If non-zero, use the SDL_GameController interface instead of the
+    // SDL_Joystick interface.
+    //
+
+    CONFIG_VARIABLE_INT(use_gamepad),
+
+    //!
+    // Stores the SDL_GameControllerType of the last configured gamepad.
+    //
+
+    CONFIG_VARIABLE_INT(gamepad_type),
+
+    //!
+    // Joystick x axis dead zone, specified as a percentage of the axis max
+    // value.
+    //
+
+    CONFIG_VARIABLE_INT(joystick_x_dead_zone),
+
+    //!
+    // Joystick y axis dead zone, specified as a percentage of the axis max
+    // value.
+    //
+
+    CONFIG_VARIABLE_INT(joystick_y_dead_zone),
+
+    //!
+    // Joystick strafe axis dead zone, specified as a percentage of the axis
+    // max value.
+    //
+
+    CONFIG_VARIABLE_INT(joystick_strafe_dead_zone),
+
+    //!
+    // Joystick look axis dead zone, specified as a percentage of the axis max
+    // value.
+    //
+
+    CONFIG_VARIABLE_INT(joystick_look_dead_zone),
+
     //!
     // Joystick virtual button to make the player strafe left.
     //
@@ -1539,6 +1648,48 @@ static default_t extra_defaults_list[] =
 
     CONFIG_VARIABLE_INT(joyb_nextweapon),
 
+    //!
+    // @game heretic hexen
+    // Joystick virtual button to activate artifact.
+    //
+
+    CONFIG_VARIABLE_INT(joyb_useartifact),
+
+    //!
+    // @game heretic hexen
+    // Joystick virtual button to move left in the inventory.
+    //
+
+    CONFIG_VARIABLE_INT(joyb_invleft),
+
+    //!
+    // @game heretic hexen
+    // Joystick virtual button to move right in the inventory.
+    //
+
+    CONFIG_VARIABLE_INT(joyb_invright),
+
+    //!
+    // @game heretic hexen
+    // Joystick virtual button to fly up.
+    //
+
+    CONFIG_VARIABLE_INT(joyb_flyup),
+
+    //!
+    // @game heretic hexen
+    // Joystick virtual button to fly down.
+    //
+
+    CONFIG_VARIABLE_INT(joyb_flydown),
+
+    //!
+    // @game heretic hexen
+    // Joystick virtual button to center flying.
+    //
+
+    CONFIG_VARIABLE_INT(joyb_flycenter),
+
     //!
     // Key to pause or unpause the game.
     //
@@ -2303,6 +2454,14 @@ static default_t extra_defaults_list[] =
 
     CONFIG_VARIABLE_INT(crispy_freelook),
 
+    //!
+    // @game doom heretic hexen strife
+    //
+    // Crispy's variable to support intermediate and negative gamma levels.
+    //
+
+    CONFIG_VARIABLE_INT(crispy_gamma),
+
     //!
     // @game doom
     //
