@@ -17,12 +17,14 @@
 //
 
 #include <stdlib.h>
+#include <string.h>
 
 #include "config.h"
 #include "d_iwad.h"
 #include "i_glob.h"
 #include "i_system.h"
 #include "m_argv.h"
+#include "m_misc.h"
 #include "w_main.h"
 #include "w_merge.h"
 #include "w_wad.h"
@@ -204,9 +206,16 @@ boolean W_ParseCommandLine(void)
 
 // Load all WAD files from the given directory.
 void W_AutoLoadWADs(const char *path)
+{
+    W_AutoLoadWADsRename(path, NULL, 0);
+}
+
+void W_AutoLoadWADsRename(const char *path, const lump_rename_t *renames,
+                          int num_renames)
 {
     glob_t *glob;
-    const char *filename;
+    const char *filename, *basename;
+    int i, j;
 
     glob = I_StartMultiGlob(path, GLOB_FLAG_NOCASE|GLOB_FLAG_SORTED,
                             "*.wad", "*.lmp", NULL);
@@ -219,6 +228,21 @@ void W_AutoLoadWADs(const char *path)
         }
         printf(" [autoload] merging %s\n", filename);
         W_MergeFile(filename);
+
+        if (renames && num_renames > 0)
+        {
+            basename = M_BaseName(filename);
+            for (i = 0; i < num_renames; i++)
+            {
+                j = W_CheckNumForName(renames[i].name);
+
+                if (j != -1 &&
+                    !strcasecmp(W_WadNameForLump(lumpinfo[j]), basename))
+                {
+                    memcpy(lumpinfo[j]->name, renames[i].new_name, 8);
+                }
+            }
+        }
     }
 
     I_EndGlob(glob);
